@@ -1,14 +1,16 @@
 /*******************************************************************************
- * Copyright (c) 2008, 2019 Sonatype Inc. and others.
- * All rights reserved. This program and the accompanying materials
- * are made available under the terms of the Eclipse Public License v1.0
+ * Copyright (c) 2008, 2020 Sonatype Inc. and others.
+ * This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License 2.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/epl-v10.html
+ * https://www.eclipse.org/legal/epl-2.0/
+ *
+ * SPDX-License-Identifier: EPL-2.0
  *
  * Contributors:
  *    Sonatype Inc. - initial API and implementation
  *    SAP SE - apply DRY principle
- *    Christoph Läubrich - adjust to changed Resolver API
+ *    Christoph Läubrich - adjust to changed API
  *******************************************************************************/
 package org.eclipse.tycho.p2.resolver;
 
@@ -22,28 +24,30 @@ import static org.eclipse.tycho.p2.target.ExecutionEnvironmentTestUtils.NOOP_EE_
 import static org.eclipse.tycho.p2.target.ExecutionEnvironmentTestUtils.customEEResolutionHintProvider;
 import static org.eclipse.tycho.p2.target.ExecutionEnvironmentTestUtils.standardEEResolutionHintProvider;
 import static org.eclipse.tycho.p2.testutil.InstallableUnitMatchers.unitWithId;
-import static org.hamcrest.CoreMatchers.containsString;
 import static org.hamcrest.CoreMatchers.hasItem;
 import static org.hamcrest.CoreMatchers.is;
 import static org.hamcrest.CoreMatchers.not;
+import static org.hamcrest.MatcherAssert.assertThat;
 import static org.junit.Assert.assertEquals;
 import static org.junit.Assert.assertFalse;
-import static org.junit.Assert.assertThat;
+import static org.junit.Assert.assertThrows;
+import static org.junit.Assert.assertTrue;
 import static org.junit.Assert.fail;
 
 import java.io.File;
 import java.io.IOException;
 import java.util.ArrayList;
 import java.util.Collections;
-import java.util.Comparator;
 import java.util.HashSet;
 import java.util.List;
 import java.util.Map;
+import java.util.Objects;
 import java.util.Set;
 
 import org.eclipse.equinox.p2.metadata.IInstallableUnit;
 import org.eclipse.tycho.ArtifactType;
 import org.eclipse.tycho.ReactorProject;
+import org.eclipse.tycho.core.ee.shared.ExecutionEnvironmentStub;
 import org.eclipse.tycho.core.resolver.shared.OptionalResolutionAction;
 import org.eclipse.tycho.core.shared.TargetEnvironment;
 import org.eclipse.tycho.p2.impl.publisher.DependencyMetadata;
@@ -63,14 +67,11 @@ import org.hamcrest.TypeSafeMatcher;
 import org.junit.Before;
 import org.junit.Rule;
 import org.junit.Test;
-import org.junit.rules.ExpectedException;
 
 public class P2ResolverTest extends P2ResolverTestBase {
 
     @Rule
     public final LogVerifier logVerifier = new LogVerifier();
-    @Rule
-    public ExpectedException expectedException = ExpectedException.none();
 
     private ReactorProject projectToResolve;
     private P2ResolutionResult result;
@@ -78,7 +79,7 @@ public class P2ResolverTest extends P2ResolverTestBase {
     @Before
     public void initDefaultResolver() throws Exception {
 //        org.eclipse.equinox.internal.p2.core.helpers.Tracing.DEBUG_PLANNER_PROJECTOR = true;
-        pomDependencies = resolverFactory.newPomDependencyCollectorImpl();
+        pomDependencies = resolverFactory.newPomDependencyCollector();
         impl = new P2ResolverImpl(tpFactory, logVerifier.getLogger());
         impl.setEnvironments(getEnvironments());
     }
@@ -193,18 +194,12 @@ public class P2ResolverTest extends P2ResolverTestBase {
 
         assertEquals(3, result.getArtifacts().size());
         List<P2ResolutionResult.Entry> entries = new ArrayList<>(result.getArtifacts());
-        Collections.sort(entries, new Comparator<Entry>() {
-
-            @Override
-            public int compare(Entry entry1, Entry entry2) {
-                return entry1.getId().compareTo(entry2.getId());
-            }
-        });
+        Collections.sort(entries, (entry1, entry2) -> entry1.getId().compareTo(entry2.getId()));
         assertEquals("org.eclipse.tycho.p2.impl.resolver.test.bundle01", entries.get(0).getId());
         assertEquals("org.eclipse.tycho.p2.impl.resolver.test.bundle01.source", entries.get(1).getId());
         assertEquals("org.eclipse.tycho.p2.impl.resolver.test.feature01", entries.get(2).getId());
-        assertEquals(bundle, entries.get(0).getLocation());
-        assertEquals(bundle, entries.get(1).getLocation());
+        assertEquals(bundle, entries.get(0).getLocation(true));
+        assertEquals(bundle, entries.get(1).getLocation(true));
         assertEquals("sources", entries.get(1).getClassifier());
     }
 
@@ -336,14 +331,15 @@ public class P2ResolverTest extends P2ResolverTestBase {
         projectToResolve = createReactorProject(resourceFile("resolver/bundle.bree"), TYPE_ECLIPSE_PLUGIN, artifactId);
 
         result = singleEnv(impl.resolveTargetDependencies(
-                getTargetPlatform(standardEEResolutionHintProvider("CDC-1.0/Foundation-1.0")), projectToResolve));
+                getTargetPlatform(
+                        standardEEResolutionHintProvider(new ExecutionEnvironmentStub("CDC-1.0/Foundation-1.0"))),
+                projectToResolve));
 
         assertEquals(2, result.getArtifacts().size());
 
-        assertEquals(3, result.getNonReactorUnits().size());
+        assertEquals(2, result.getNonReactorUnits().size());
         assertContainsUnit("javax.xml", result.getNonReactorUnits());
         assertContainsUnit("a.jre.cdc", result.getNonReactorUnits());
-        assertContainsUnit("config.a.jre.cdc", result.getNonReactorUnits());
     }
 
     @Test
@@ -354,13 +350,14 @@ public class P2ResolverTest extends P2ResolverTestBase {
         projectToResolve = createReactorProject(resourceFile("resolver/bundle.bree"), TYPE_ECLIPSE_PLUGIN, artifactId);
 
         result = singleEnv(impl.resolveTargetDependencies(
-                getTargetPlatform(standardEEResolutionHintProvider("J2SE-1.5")), projectToResolve));
+                getTargetPlatform(
+                        standardEEResolutionHintProvider(new ExecutionEnvironmentStub("J2SE-1.5", "org.w3c.dom"))),
+                projectToResolve));
 
         assertEquals(1, result.getArtifacts().size());
 
-        assertEquals(2, result.getNonReactorUnits().size());
+        assertEquals(1, result.getNonReactorUnits().size());
         assertContainsUnit("a.jre.j2se", result.getNonReactorUnits());
-        assertContainsUnit("config.a.jre.j2se", result.getNonReactorUnits());
     }
 
     @Test
@@ -401,12 +398,14 @@ public class P2ResolverTest extends P2ResolverTestBase {
                 TYPE_ECLIPSE_FEATURE, "p2Inf.conflicting-main-artifact");
 
         // bug 430728: explicit detect this conflict and throw an exception
-        expectedException.expectMessage("classifier");
-        result = singleEnv(impl.resolveTargetDependencies(getTargetPlatform(), projectToResolve));
+        Exception e = assertThrows(Exception.class, () -> {
+            result = singleEnv(impl.resolveTargetDependencies(getTargetPlatform(), projectToResolve));
+            // ... or the p2.inf "artifact" could also just be omitted
+            assertThat(getClassifiedArtifact(result, null).getType(), is(ArtifactType.TYPE_ECLIPSE_FEATURE));
+            assertThat(result.getArtifacts().size(), is(1));
+        });
+        assertTrue(e.getMessage().contains("classifier"));
 
-        // ... or the p2.inf "artifact" could also just be omitted
-        assertThat(getClassifiedArtifact(result, null).getType(), is(ArtifactType.TYPE_ECLIPSE_FEATURE));
-        assertThat(result.getArtifacts().size(), is(1));
     }
 
     @Test
@@ -506,19 +505,6 @@ public class P2ResolverTest extends P2ResolverTestBase {
         assertThat((Set<IInstallableUnit>) result.getNonReactorUnits(), hasItem(unitWithId("org.eclipse.osgi")));
     }
 
-    @Test
-    public void testMissingArtifact() throws Exception {
-        // repository with the IU org.eclipse.osgi but not the corresponding artifact (-> this repository is inconsistent, more often you'd get this situation in offline mode)
-        tpConfig.addP2Repository(resourceFile("repositories/missing-artifact").toURI());
-        // module requiring org.eclipse.osgi
-        projectToResolve = createReactorProject(resourceFile("resolver/bundle01"), TYPE_ECLIPSE_PLUGIN,
-                "org.eclipse.tycho.p2.impl.resolver.test.bundle01");
-
-        expectedException.expectMessage("could not be downloaded");
-        logVerifier.expectError(containsString("could not be downloaded"));
-        impl.resolveTargetDependencies(getTargetPlatform(), projectToResolve);
-    }
-
     private P2TargetPlatform getTargetPlatform() {
         return tpFactory.createTargetPlatform(tpConfig, NOOP_EE_RESOLUTION_HANDLER, reactorProjects, pomDependencies);
     }
@@ -538,7 +524,7 @@ public class P2ResolverTest extends P2ResolverTestBase {
         P2ResolutionResult.Entry selectedEntry = null;
         for (Entry entry : resolutionResult.getArtifacts()) {
             availableClassifiers.add(entry.getClassifier());
-            if (eq(classifier, entry.getClassifier())) {
+            if (Objects.equals(classifier, entry.getClassifier())) {
                 selectedEntry = entry;
             }
         }
@@ -578,21 +564,11 @@ public class P2ResolverTest extends P2ResolverTestBase {
 
     private static void assertContainLocation(P2ResolutionResult result, File location) {
         for (P2ResolutionResult.Entry entry : result.getArtifacts()) {
-            if (entry.getLocation().equals(location)) {
+            if (entry.getLocation(true).equals(location)) {
                 return;
             }
         }
         fail();
     }
 
-    private static boolean eq(String left, String right) {
-        if (left == right) {
-            return true;
-        } else if (left == null) {
-            return false;
-        } else {
-            return left.equals(right);
-        }
-    }
-
 }
