@@ -1,9 +1,11 @@
 /*******************************************************************************
  * Copyright (c) 2008, 2015 Sonatype Inc. and others.
- * All rights reserved. This program and the accompanying materials
- * are made available under the terms of the Eclipse Public License v1.0
+ * This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License 2.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/epl-v10.html
+ * https://www.eclipse.org/legal/epl-2.0/
+ *
+ * SPDX-License-Identifier: EPL-2.0
  *
  * Contributors:
  *    Sonatype Inc. - initial API and implementation
@@ -23,8 +25,8 @@ import java.util.LinkedHashMap;
 import java.util.LinkedHashSet;
 import java.util.List;
 import java.util.Map;
+import java.util.Optional;
 import java.util.Set;
-import java.util.TreeSet;
 
 import org.eclipse.core.runtime.IProgressMonitor;
 import org.eclipse.equinox.internal.p2.metadata.InstallableUnit;
@@ -40,13 +42,16 @@ import org.eclipse.equinox.p2.publisher.eclipse.BundlesAction;
 import org.eclipse.equinox.p2.query.IQueryResult;
 import org.eclipse.equinox.p2.query.QueryUtil;
 import org.eclipse.equinox.spi.p2.publisher.PublisherHelper;
+import org.eclipse.tycho.ArtifactKey;
 import org.eclipse.tycho.ArtifactType;
+import org.eclipse.tycho.DefaultArtifactKey;
 import org.eclipse.tycho.PackagingType;
 import org.eclipse.tycho.ReactorProject;
 import org.eclipse.tycho.ReactorProjectIdentities;
 import org.eclipse.tycho.artifacts.DependencyResolutionException;
 import org.eclipse.tycho.artifacts.IllegalArtifactReferenceException;
 import org.eclipse.tycho.artifacts.TargetPlatform;
+import org.eclipse.tycho.core.ee.shared.ExecutionEnvironmentConfiguration;
 import org.eclipse.tycho.core.ee.shared.ExecutionEnvironmentConfigurationStub;
 import org.eclipse.tycho.core.shared.MavenLogger;
 import org.eclipse.tycho.core.shared.MultiLineLogger;
@@ -171,13 +176,14 @@ public class P2ResolverImpl implements P2Resolver {
     }
 
     @Override
-    public P2ResolutionResult resolveMetadata(TargetPlatformConfigurationStub tpConfiguration, String eeName) {
-        P2TargetPlatform contextImpl = targetPlatformFactory.createTargetPlatform(tpConfiguration,
-                new ExecutionEnvironmentConfigurationStub(eeName), null, null);
+    public P2ResolutionResult resolveMetadata(TargetPlatformConfigurationStub tpConfiguration,
+            ExecutionEnvironmentConfiguration eeConfig) {
+        P2TargetPlatform contextImpl = targetPlatformFactory.createTargetPlatform(tpConfiguration, eeConfig, null,
+                null);
 
         ResolutionDataImpl data = new ResolutionDataImpl(contextImpl.getEEResolutionHints());
         data.setAvailableIUs(contextImpl.getInstallableUnits());
-        data.setRootIUs(new HashSet<IInstallableUnit>());
+        data.setRootIUs(new HashSet<>());
         data.setAdditionalRequirements(additionalRequirements);
 
         ProjectorResolutionStrategy strategy = new ProjectorResolutionStrategy(logger);
@@ -196,6 +202,11 @@ public class P2ResolverImpl implements P2Resolver {
         return result;
     }
 
+    @Override
+    public P2ResolutionResult resolveMetadata(TargetPlatformConfigurationStub tpConfiguration, String eeName) {
+        return resolveMetadata(tpConfiguration, new ExecutionEnvironmentConfigurationStub(eeName));
+    }
+
     // TODO 412416 make this obsolete by adding appropriate getters in TargetPlatform interface
     @Override
     public P2ResolutionResult getTargetPlatformAsResolutionResult(TargetPlatformConfigurationStub tpConfiguration,
@@ -240,7 +251,7 @@ public class P2ResolverImpl implements P2Resolver {
             logger.error("Cannot resolve project dependencies:");
             new MultiLineLogger(logger).error(e.getDetails(), "  ");
             logger.error("");
-            logger.error("See http://wiki.eclipse.org/Tycho/Dependency_Resolution_Troubleshooting for help.");
+            logger.error("See https://wiki.eclipse.org/Tycho/Dependency_Resolution_Troubleshooting for help.");
             throw new DependencyResolutionException("Cannot resolve dependencies of " + project, e);
         }
 
@@ -254,10 +265,9 @@ public class P2ResolverImpl implements P2Resolver {
     private P2ResolutionResult toResolutionResult(Collection<IInstallableUnit> newState,
             ReactorProject currentProject) {
         DefaultP2ResolutionResult result = new DefaultP2ResolutionResult();
-        Set<String> missingArtifacts = new TreeSet<>();
 
         for (IInstallableUnit iu : newState) {
-            addUnit(result, iu, currentProject, missingArtifacts);
+            addUnit(result, iu, currentProject);
         }
         // remove entries for which there were only "additional" IUs, but none with a recognized type
         result.removeEntriesWithUnknownType();
@@ -265,15 +275,12 @@ public class P2ResolverImpl implements P2Resolver {
         // local repository index needs to be saved manually
         context.saveLocalMavenRepository();
 
-        failIfArtifactsMissing(missingArtifacts);
-
         // TODO 372780 remove; no longer needed when aggregation uses frozen target platform as source
         collectNonReactorIUs(result, newState);
         return result;
     }
 
-    private void addUnit(DefaultP2ResolutionResult result, IInstallableUnit iu, ReactorProject currentProject,
-            Set<String> missingArtifacts) {
+    private void addUnit(DefaultP2ResolutionResult result, IInstallableUnit iu, ReactorProject currentProject) {
 
         if (currentProjectUnits.contains(iu)) {
             addReactorProject(result, currentProject.getIdentities(), iu);
@@ -293,25 +300,7 @@ public class P2ResolverImpl implements P2Resolver {
         }
 
         for (IArtifactKey key : iu.getArtifacts()) {
-            // this downloads artifacts if necessary; TODO parallelize download?
-            File artifactLocation = context.getLocalArtifactFile(key);
-
-            if (artifactLocation == null) {
-                missingArtifacts.add(key.toString());
-            } else {
-                addArtifactFile(result, iu, key, artifactLocation);
-            }
-        }
-    }
-
-    private void failIfArtifactsMissing(Set<String> missingArtifacts) {
-        if (!missingArtifacts.isEmpty()) {
-            logger.error("The following artifacts could not be downloaded: ");
-            for (String missingArtifact : missingArtifacts) {
-                logger.error("  " + missingArtifact);
-            }
-            // TODO throw a typed exception here, so that we can log more information depending on the offline mode further up in the call stack
-            throw new RuntimeException("Some required artifacts could not be downloaded. See log output for details.");
+            addArtifactFile(result, iu, key, context);
         }
     }
 
@@ -325,20 +314,24 @@ public class P2ResolverImpl implements P2Resolver {
         }
     }
 
-    private void addArtifactFile(DefaultP2ResolutionResult result, IInstallableUnit iu, IArtifactKey key,
-            File artifactLocation) {
+    private void addArtifactFile(DefaultP2ResolutionResult result, IInstallableUnit iu, IArtifactKey p2ArtifactKey,
+            P2TargetPlatform context) {
         String id = iu.getId();
         String version = iu.getVersion().toString();
         String mavenClassifier = null;
 
-        if (PublisherHelper.OSGI_BUNDLE_CLASSIFIER.equals(key.getClassifier())) {
-            result.addArtifact(ArtifactType.TYPE_ECLIPSE_PLUGIN, id, version, artifactLocation, mavenClassifier, iu);
-        } else if (PublisherHelper.ECLIPSE_FEATURE_CLASSIFIER.equals(key.getClassifier())) {
+        if (PublisherHelper.OSGI_BUNDLE_CLASSIFIER.equals(p2ArtifactKey.getClassifier())) {
+            ArtifactKey artifactKey = new DefaultArtifactKey(ArtifactType.TYPE_ECLIPSE_PLUGIN, id, version);
+            result.addArtifact(artifactKey, mavenClassifier, iu, p2ArtifactKey, context);
+        } else if (PublisherHelper.ECLIPSE_FEATURE_CLASSIFIER.equals(p2ArtifactKey.getClassifier())) {
             String featureId = getFeatureId(iu);
             if (featureId != null) {
-                result.addArtifact(ArtifactType.TYPE_ECLIPSE_FEATURE, featureId, version, artifactLocation,
-                        mavenClassifier, iu);
+                ArtifactKey artifactKey = new DefaultArtifactKey(ArtifactType.TYPE_ECLIPSE_FEATURE, featureId, version);
+                result.addArtifact(artifactKey, mavenClassifier, iu, p2ArtifactKey, context);
             }
+        } else {
+            ArtifactKey key = new DefaultArtifactKey(ArtifactType.TYPE_INSTALLABLE_UNIT, id, version);
+            result.addArtifact(key, mavenClassifier, iu, p2ArtifactKey, context);
         }
 
         // ignore other/unknown artifacts, like binary blobs for now.
@@ -393,13 +386,14 @@ public class P2ResolverImpl implements P2Resolver {
             }
         }
 
-        result.addArtifact(contributingArtifactType, contributingArtifactId, version, location, mavenClassifier, iu);
+        result.addResolvedArtifact(
+                Optional.ofNullable(contributingArtifactId)
+                        .map(artifactId -> new DefaultArtifactKey(contributingArtifactType, artifactId, version)),
+                mavenClassifier, iu, location);
     }
 
     private boolean isPureIU(IInstallableUnit iu) {
-        if ("true".equals(iu.getProperty(AuthoredIUAction.IU_TYPE)))
-            return true;
-        return false;
+        return Boolean.parseBoolean(iu.getProperty(AuthoredIUAction.IU_TYPE));
     }
 
     private String getFeatureId(IInstallableUnit iu) {
