@@ -1,9 +1,11 @@
 /*******************************************************************************
  * Copyright (c) 2013 SAP AG and others.
- * All rights reserved. This program and the accompanying materials
- * are made available under the terms of the Eclipse Public License v1.0
+ * This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License 2.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/epl-v10.html
+ * https://www.eclipse.org/legal/epl-2.0/
+ *
+ * SPDX-License-Identifier: EPL-2.0
  *
  * Contributors:
  *    SAP AG - initial API and implementation
@@ -19,11 +21,6 @@ import org.eclipse.tycho.core.ee.shared.ExecutionEnvironmentConfiguration;
 // TODO 412416 javadoc
 public interface TargetPlatformFactory {
 
-    /**
-     * Creates a new object for collecting the bundles within the POM dependencies.
-     */
-    public PomDependencyCollector newPomDependencyCollector();
-
     public TargetPlatform createTargetPlatform(TargetPlatformConfigurationStub tpConfiguration,
             ExecutionEnvironmentConfiguration eeConfiguration, List<ReactorProject> reactorProjects,
             PomDependencyCollector pomDependencies);
