@@ -18,13 +18,20 @@ package org.eclipse.tycho.compiler;
 
 import java.io.File;
 import java.io.IOException;
+import java.nio.file.Path;
+import java.nio.file.Paths;
 import java.util.ArrayList;
 import java.util.Arrays;
+import java.util.Collection;
 import java.util.Collections;
+import java.util.Comparator;
 import java.util.HashSet;
 import java.util.List;
 import java.util.Map;
+import java.util.Objects;
+import java.util.Optional;
 import java.util.Set;
+import java.util.stream.Collectors;
 
 import org.apache.maven.artifact.Artifact;
 import org.apache.maven.artifact.repository.ArtifactRepository;
@@ -39,6 +46,7 @@ import org.apache.maven.plugins.annotations.Parameter;
 import org.apache.maven.project.MavenProject;
 import org.apache.maven.project.artifact.ProjectArtifact;
 import org.apache.maven.repository.RepositorySystem;
+import org.apache.maven.toolchain.ToolchainManager;
 import org.apache.maven.toolchain.ToolchainManagerPrivate;
 import org.apache.maven.toolchain.java.DefaultJavaToolChain;
 import org.codehaus.plexus.compiler.CompilerConfiguration;
@@ -46,9 +54,13 @@ import org.codehaus.plexus.compiler.util.scan.InclusionScanException;
 import org.codehaus.plexus.compiler.util.scan.SimpleSourceInclusionScanner;
 import org.codehaus.plexus.compiler.util.scan.SourceInclusionScanner;
 import org.codehaus.plexus.compiler.util.scan.StaleSourceScanner;
+import org.codehaus.plexus.logging.Logger;
 import org.codehaus.plexus.util.DirectoryScanner;
 import org.codehaus.plexus.util.FileUtils;
 import org.codehaus.plexus.util.xml.Xpp3Dom;
+import org.eclipse.jdt.internal.compiler.util.CtSym;
+import org.eclipse.jdt.internal.compiler.util.JRTUtil;
+import org.eclipse.osgi.util.ManifestElement;
 import org.eclipse.tycho.classpath.ClasspathEntry;
 import org.eclipse.tycho.classpath.ClasspathEntry.AccessRule;
 import org.eclipse.tycho.classpath.JavaCompilerConfiguration;
@@ -56,6 +68,7 @@ import org.eclipse.tycho.classpath.SourcepathEntry;
 import org.eclipse.tycho.core.BundleProject;
 import org.eclipse.tycho.core.TychoConstants;
 import org.eclipse.tycho.core.TychoProject;
+import org.eclipse.tycho.core.ee.ExecutionEnvironmentUtils;
 import org.eclipse.tycho.core.ee.StandardExecutionEnvironment;
 import org.eclipse.tycho.core.ee.shared.ExecutionEnvironment;
 import org.eclipse.tycho.core.maven.ToolchainProvider;
@@ -65,15 +78,23 @@ import org.eclipse.tycho.core.osgitools.DefaultClasspathEntry;
 import org.eclipse.tycho.core.osgitools.DefaultClasspathEntry.DefaultAccessRule;
 import org.eclipse.tycho.core.osgitools.DefaultReactorProject;
 import org.eclipse.tycho.core.osgitools.OsgiBundleProject;
+import org.eclipse.tycho.core.osgitools.OsgiManifest;
 import org.eclipse.tycho.core.osgitools.project.BuildOutputJar;
 import org.eclipse.tycho.core.osgitools.project.EclipsePluginProject;
 import org.eclipse.tycho.core.utils.TychoProjectUtils;
 import org.eclipse.tycho.runtime.Adaptable;
+import org.osgi.framework.Constants;
+import org.osgi.framework.Filter;
+import org.osgi.framework.FrameworkUtil;
+import org.osgi.framework.InvalidSyntaxException;
+import org.osgi.framework.Version;
+import org.osgi.framework.namespace.ExecutionEnvironmentNamespace;
+import org.osgi.resource.Namespace;
 
 import copied.org.apache.maven.plugin.AbstractCompilerMojo;
 
-public abstract class AbstractOsgiCompilerMojo extends AbstractCompilerMojo implements JavaCompilerConfiguration,
-        Adaptable {
+public abstract class AbstractOsgiCompilerMojo extends AbstractCompilerMojo
+        implements JavaCompilerConfiguration, Adaptable {
 
     public static final String RULE_SEPARATOR = File.pathSeparator;
 
@@ -82,6 +103,11 @@ public abstract class AbstractOsgiCompilerMojo extends AbstractCompilerMojo impl
      */
     public static final String RULE_EXCLUDE_ALL = "?**/*";
 
+    /**
+     * Lock object to ensure thread-safety
+     */
+    private static final Object LOCK = new Object();
+
     private static final Set<String> MATCH_ALL = Collections.singleton("**/*");
 
     private static final String PREFS_FILE_PATH = ".settings" + File.separator + "org.eclipse.jdt.core.prefs";
@@ -110,8 +136,8 @@ public abstract class AbstractOsgiCompilerMojo extends AbstractCompilerMojo impl
     /**
      * Which JDK to use for compilation. Default value is SYSTEM which means the currently running
      * JDK. If BREE is specified, MANIFEST header <code>Bundle-RequiredExecutionEnvironment</code>
-     * is used to define the JDK to compile against. In this case, you need to provide a <a
-     * href="http://maven.apache.org/guides/mini/guide-using-toolchains.html">toolchains.xml</a>
+     * is used to define the JDK to compile against. In this case, you need to provide a
+     * <a href="http://maven.apache.org/guides/mini/guide-using-toolchains.html">toolchains.xml</a>
      * configuration file. The value of BREE will be matched against the id of the toolchain
      * elements in toolchains.xml. Example:
      * 
@@ -179,8 +205,8 @@ public abstract class AbstractOsgiCompilerMojo extends AbstractCompilerMojo impl
 
     /**
      * Whether a bundle is required to explicitly import non-java.* packages from the JDK. This is
-     * the design-time equivalent to the equinox runtime option <a
-     * href="http://wiki.eclipse.org/Equinox_Boot_Delegation#The_solution"
+     * the design-time equivalent to the equinox runtime option
+     * <a href="https://wiki.eclipse.org/Equinox_Boot_Delegation#The_solution"
      * >osgi.compatibility.bootdelegation</a>.
      */
     @Parameter(defaultValue = "false")
@@ -202,6 +228,18 @@ public abstract class AbstractOsgiCompilerMojo extends AbstractCompilerMojo impl
     @Parameter(defaultValue = "true")
     private boolean useProjectSettings;
 
+    /**
+     * Whether the <code>-release</code> argument for the Java compiler should be derived from the
+     * target level. Enabled by default.
+     * <p>
+     * Disabling this can be useful in situations where compiling using <code>-release</code> cannot
+     * be used, e.g. when referencing internal JDK classes exported via an OSGI framework extension.
+     * In that case <code>&lt;release&gt;</code> should also be explicitly set to an empty value to
+     * prevent it from being inherited.
+     */
+    @Parameter(defaultValue = "true")
+    private boolean deriveReleaseCompilerArgumentFromTargetLevel = true;
+
     /**
      * Current build output jar
      */
@@ -224,10 +262,10 @@ public abstract class AbstractOsgiCompilerMojo extends AbstractCompilerMojo impl
      * <code>${project.build.outputDirectory}</code>.
      * 
      * Set this to <code>false</code> in case you want to keep resources separate from java files in
-     * <code>src/main/resources</code> and handle them using <a
-     * href="http://maven.apache.org/plugins/maven-resources-plugin/"> maven-resources-plugin</a>
-     * (e.g. for <a
-     * href="http://maven.apache.org/plugins/maven-resources-plugin/examples/filter.html">resource
+     * <code>src/main/resources</code> and handle them using
+     * <a href="http://maven.apache.org/plugins/maven-resources-plugin/"> maven-resources-plugin</a>
+     * (e.g. for <a href=
+     * "http://maven.apache.org/plugins/maven-resources-plugin/examples/filter.html">resource
      * filtering<a/>.
      * 
      */
@@ -276,31 +314,79 @@ public abstract class AbstractOsgiCompilerMojo extends AbstractCompilerMojo impl
     @Component
     ToolchainProvider toolchainProvider;
 
+    @Component
+    private ToolchainManager toolchainManager;
+
+    @Component
+    private Logger logger;
+
+    private StandardExecutionEnvironment[] manifestBREEs;
+
     @Override
     public void execute() throws MojoExecutionException, MojoFailureException {
-        StandardExecutionEnvironment[] manifestBREEs = bundleReader.loadManifest(project.getBasedir())
-                .getExecutionEnvironments();
-        getLog().debug("Manifest BREEs: " + Arrays.toString(manifestBREEs));
-        getLog().debug("Effective EE: " + getTargetExecutionEnvironment());
+        getLog().debug("Manifest BREEs: " + Arrays.toString(getBREE()));
+        getLog().debug("Target Platform EE: " + getTargetExecutionEnvironment());
         String effectiveTargetLevel = getTargetLevel();
         getLog().debug("Effective source/target: " + getSourceLevel() + "/" + effectiveTargetLevel);
 
         checkTargetLevelCompatibleWithManifestBREEs(effectiveTargetLevel, manifestBREEs);
 
-        for (BuildOutputJar jar : getEclipsePluginProject().getOutputJars()) {
-            this.outputJar = jar;
-            this.outputJar.getOutputDirectory().mkdirs();
-            super.execute();
-            doCopyResources();
-        }
+        synchronized (LOCK) {
+            for (BuildOutputJar jar : getEclipsePluginProject().getOutputJars()) {
+                this.outputJar = jar;
+                this.outputJar.getOutputDirectory().mkdirs();
+                super.execute();
+                doCopyResources();
+            }
 
-        // this does not include classes from nested jars
-        BuildOutputJar dotOutputJar = getEclipsePluginProject().getDotOutputJar();
-        if (dotOutputJar != null) {
-            project.getArtifact().setFile(dotOutputJar.getOutputDirectory());
+            // this does not include classes from nested jars
+            BuildOutputJar dotOutputJar = getEclipsePluginProject().getDotOutputJar();
+            if (dotOutputJar != null) {
+                project.getArtifact().setFile(dotOutputJar.getOutputDirectory());
+            }
         }
     }
 
+    /**
+     * Only public for tests purpose!
+     */
+    public StandardExecutionEnvironment[] getBREE() {
+        if (manifestBREEs == null) {
+            OsgiManifest manifest = bundleReader.loadManifest(project.getBasedir());
+            manifestBREEs = Arrays.stream(manifest.getExecutionEnvironments())
+                    .map(ee -> ExecutionEnvironmentUtils.getExecutionEnvironment(ee, toolchainManager, session, logger))
+                    .toArray(StandardExecutionEnvironment[]::new);
+            if (manifestBREEs.length == 0) {
+                ManifestElement[] requireCapability = manifest.getManifestElements(Constants.REQUIRE_CAPABILITY);
+                if (requireCapability != null) {
+                    List<Filter> eeFilters = Arrays.stream(requireCapability)
+                            .filter(element -> ExecutionEnvironmentNamespace.EXECUTION_ENVIRONMENT_NAMESPACE
+                                    .equals(element.getValue())) //
+                            .map(element -> element.getDirective(Namespace.REQUIREMENT_FILTER_DIRECTIVE)) //
+                            .map(filterDirective -> {
+                                try {
+                                    return FrameworkUtil.createFilter(filterDirective);
+                                } catch (InvalidSyntaxException e) {
+                                    e.printStackTrace();
+                                    return null;
+                                }
+                            }).filter(Objects::nonNull).collect(Collectors.toList());
+                    manifestBREEs = ExecutionEnvironmentUtils.getProfileNames().stream() //
+                            .map(name -> name.split("-")) //
+                            .map(segments -> Map.of(ExecutionEnvironmentNamespace.EXECUTION_ENVIRONMENT_NAMESPACE,
+                                    segments[0], "version", segments[1]))
+                            .filter(eeCapability -> eeFilters.stream().anyMatch(filter -> filter.matches(eeCapability)))
+                            .map(ee -> ee.get(ExecutionEnvironmentNamespace.EXECUTION_ENVIRONMENT_NAMESPACE) + '-'
+                                    + ee.get("version"))
+                            .map(ee -> ExecutionEnvironmentUtils.getExecutionEnvironment(ee, toolchainManager, session,
+                                    logger))
+                            .toArray(StandardExecutionEnvironment[]::new);
+                }
+            }
+        }
+        return manifestBREEs;
+    }
+
     /*
      * mimics the behavior of the PDE incremental builder which by default copies all (non-java)
      * resource files in source directories into the target folder
@@ -321,6 +407,7 @@ public abstract class AbstractOsgiCompilerMojo extends AbstractCompilerMojo impl
 
             Set<String> excludes = new HashSet<>();
             excludes.addAll(excludeResources);
+            excludes.addAll(getCompileSourceExcludePaths());
             excludes.addAll(getEclipsePluginProject().getBuildProperties().getBinExcludes());
             excludes.add("**/*.java");
             StaleSourceScanner scanner = new StaleSourceScanner(0L, MATCH_ALL, excludes);
@@ -369,7 +456,7 @@ public abstract class AbstractOsgiCompilerMojo extends AbstractCompilerMojo impl
         return (BundleProject) projectType;
     }
 
-    private String toString(List<AccessRule> rules) {
+    private String toString(Collection<AccessRule> rules) {
         StringBuilder result = new StringBuilder(); // include all
         if (rules != null) {
             result.append("[");
@@ -399,6 +486,11 @@ public abstract class AbstractOsgiCompilerMojo extends AbstractCompilerMojo impl
         return roots;
     }
 
+    @Override
+    protected final List<String> getCompileSourceExcludePaths() throws MojoExecutionException {
+        return Collections.unmodifiableList(outputJar.getFilesToExclude());
+    }
+
     @Override
     public List<SourcepathEntry> getSourcepath() throws MojoExecutionException {
         ArrayList<SourcepathEntry> entries = new ArrayList<>();
@@ -465,15 +557,15 @@ public abstract class AbstractOsgiCompilerMojo extends AbstractCompilerMojo impl
     }
 
     @Override
-    protected CompilerConfiguration getCompilerConfiguration(List<String> compileSourceRoots)
-            throws MojoExecutionException, MojoFailureException {
-        CompilerConfiguration compilerConfiguration = super.getCompilerConfiguration(compileSourceRoots);
+    protected CompilerConfiguration getCompilerConfiguration(List<String> compileSourceRoots,
+            List<String> compileSourceExcludes) throws MojoExecutionException, MojoFailureException {
+        CompilerConfiguration compilerConfiguration = super.getCompilerConfiguration(compileSourceRoots,
+                compileSourceExcludes);
         if (useProjectSettings) {
             String prefsFilePath = project.getBasedir() + File.separator + PREFS_FILE_PATH;
             if (!new File(prefsFilePath).exists()) {
-                getLog().warn(
-                        "Parameter 'useProjectSettings' is set to true, but preferences file '" + prefsFilePath
-                                + "' could not be found!");
+                getLog().warn("Parameter 'useProjectSettings' is set to true, but preferences file '" + prefsFilePath
+                        + "' could not be found!");
             } else {
                 compilerConfiguration.addCompilerCustomArgument("-properties", prefsFilePath);
             }
@@ -483,7 +575,12 @@ public abstract class AbstractOsgiCompilerMojo extends AbstractCompilerMojo impl
         if (encoding != null) {
             compilerConfiguration.setSourceEncoding(encoding);
         }
-        configureSourceAndTargetLevel(compilerConfiguration);
+        compilerConfiguration.setTargetVersion(getTargetLevel());
+        compilerConfiguration.setSourceVersion(getSourceLevel());
+        String releaseLevel = getReleaseLevel();
+        if (releaseLevel != null) {
+            compilerConfiguration.setReleaseVersion(releaseLevel);
+        }
         configureJavaHome(compilerConfiguration);
         configureBootclasspathAccessRules(compilerConfiguration);
         configureCompilerLog(compilerConfiguration);
@@ -525,21 +622,25 @@ public abstract class AbstractOsgiCompilerMojo extends AbstractCompilerMojo impl
             accessRules.addAll(getStrictBootClasspathAccessRules());
         } else {
             accessRules.add(new DefaultAccessRule("java/**", false));
-            for (String pkg : getTargetExecutionEnvironment().getSystemPackages()) {
-                accessRules.add(new DefaultAccessRule(pkg.trim().replace('.', '/') + "/*", false));
-            }
+            getTargetExecutionEnvironment().getSystemPackages().stream() //
+                    .map(systemPackage -> systemPackage.packageName) //
+                    .distinct() //
+                    .map(packageName -> packageName.trim().replace('.', '/') + "/*") //
+                    .map(accessRule -> new DefaultAccessRule(accessRule, false)) //
+                    .forEach(accessRules::add);
             // now add packages exported by framework extension bundles
-            accessRules.addAll(getBundleProject().getBootClasspathExtraAccessRules(project));
+            accessRules
+                    .addAll(getBundleProject().getBootClasspathExtraAccessRules(DefaultReactorProject.adapt(project)));
         }
-        if (accessRules.size() > 0) {
-            compilerConfiguration
-                    .addCompilerCustomArgument("org.osgi.framework.system.packages", toString(accessRules));
+        if (!accessRules.isEmpty()) {
+            compilerConfiguration.addCompilerCustomArgument("org.osgi.framework.system.packages",
+                    toString(accessRules));
         }
     }
 
     @SuppressWarnings("unchecked")
     private List<AccessRule> getStrictBootClasspathAccessRules() throws MojoExecutionException {
-        return (List<AccessRule>) project
+        return (List<AccessRule>) DefaultReactorProject.adapt(project)
                 .getContextValue(TychoConstants.CTX_ECLIPSE_PLUGIN_STRICT_BOOTCLASSPATH_ACCESSRULES);
     }
 
@@ -547,7 +648,16 @@ public abstract class AbstractOsgiCompilerMojo extends AbstractCompilerMojo impl
         if (useJDK != JDKUsage.BREE) {
             return;
         }
-        String toolchainId = getTargetExecutionEnvironment().getProfileName();
+        StandardExecutionEnvironment[] brees = getBREE();
+        String toolchainId = null;
+        if (brees.length > 0) {
+            toolchainId = brees[0].getProfileName();
+        } else {
+            getLog().warn(
+                    "useJDK=BREE configured, but no BREE is set in bundle. Fail back to currently running execution environment ("
+                            + getTargetExecutionEnvironment().getProfileName() + ").");
+            toolchainId = getTargetExecutionEnvironment().getProfileName();
+        }
         DefaultJavaToolChain toolChain = toolchainProvider.findMatchingJavaToolChain(session, toolchainId);
         if (toolChain == null) {
             throw new MojoExecutionException("useJDK = BREE configured, but no toolchain of type 'jdk' with id '"
@@ -557,7 +667,8 @@ public abstract class AbstractOsgiCompilerMojo extends AbstractCompilerMojo impl
         configureBootClassPath(compilerConfiguration, toolChain);
     }
 
-    private void configureBootClassPath(CompilerConfiguration compilerConfiguration, DefaultJavaToolChain javaToolChain) {
+    private void configureBootClassPath(CompilerConfiguration compilerConfiguration,
+            DefaultJavaToolChain javaToolChain) {
         Xpp3Dom config = (Xpp3Dom) javaToolChain.getModel().getConfiguration();
         if (config != null) {
             Xpp3Dom bootClassPath = config.getChild("bootClassPath");
@@ -566,10 +677,8 @@ public abstract class AbstractOsgiCompilerMojo extends AbstractCompilerMojo impl
                 if (includeParent != null) {
                     Xpp3Dom[] includes = includeParent.getChildren("include");
                     if (includes.length > 0) {
-                        compilerConfiguration.addCompilerCustomArgument(
-                                "-bootclasspath",
-                                scanBootclasspath(javaToolChain.getJavaHome(), includes,
-                                        bootClassPath.getChild("excludes")));
+                        compilerConfiguration.addCompilerCustomArgument("-bootclasspath", scanBootclasspath(
+                                javaToolChain.getJavaHome(), includes, bootClassPath.getChild("excludes")));
                     }
                 }
             }
@@ -606,23 +715,17 @@ public abstract class AbstractOsgiCompilerMojo extends AbstractCompilerMojo impl
         return values;
     }
 
-    private void configureSourceAndTargetLevel(CompilerConfiguration compilerConfiguration)
-            throws MojoExecutionException {
-        ExecutionEnvironment ee = getTargetExecutionEnvironment();
-        compilerConfiguration.setSourceVersion(getSourceLevel(ee));
-        compilerConfiguration.setTargetVersion(getTargetLevel(ee));
-    }
-
-    private ExecutionEnvironment getTargetExecutionEnvironment() throws MojoExecutionException {
+    private ExecutionEnvironment getTargetExecutionEnvironment() {
         // never null
-        return TychoProjectUtils.getExecutionEnvironmentConfiguration(project).getFullSpecification();
+        return TychoProjectUtils.getExecutionEnvironmentConfiguration(DefaultReactorProject.adapt(project))
+                .getFullSpecification();
     }
 
     @Override
     public List<ClasspathEntry> getClasspath() throws MojoExecutionException {
         TychoProject projectType = getBundleProject();
         ArrayList<ClasspathEntry> classpath = new ArrayList<>(
-                ((BundleProject) projectType).getClasspath(project));
+                ((BundleProject) projectType).getClasspath(DefaultReactorProject.adapt(project)));
 
         if (extraClasspathElements != null) {
             ArtifactRepository localRepository = session.getLocalRepository();
@@ -639,8 +742,8 @@ public abstract class AbstractOsgiCompilerMojo extends AbstractCompilerMojo impl
                 ArtifactResolutionResult result = repositorySystem.resolve(request);
 
                 if (result.hasExceptions()) {
-                    throw new MojoExecutionException("Could not resolve extra classpath entry", result.getExceptions()
-                            .get(0));
+                    throw new MojoExecutionException("Could not resolve extra classpath entry",
+                            result.getExceptions().get(0));
                 }
 
                 for (Artifact b : result.getArtifacts()) {
@@ -650,8 +753,8 @@ public abstract class AbstractOsgiCompilerMojo extends AbstractCompilerMojo impl
                     }
                     ArrayList<File> bLocations = new ArrayList<>();
                     bLocations.add(b.getFile()); // TODO properly handle multiple project locations maybe
-                    classpath.add(new DefaultClasspathEntry(DefaultReactorProject.adapt(bProject), null, bLocations,
-                            null));
+                    classpath.add(
+                            new DefaultClasspathEntry(DefaultReactorProject.adapt(bProject), null, bLocations, null));
                 }
             }
         }
@@ -665,10 +768,6 @@ public abstract class AbstractOsgiCompilerMojo extends AbstractCompilerMojo impl
 
     @Override
     public String getSourceLevel() throws MojoExecutionException {
-        return getSourceLevel(getTargetExecutionEnvironment());
-    }
-
-    private String getSourceLevel(ExecutionEnvironment ee) throws MojoExecutionException {
         // first, explicit POM configuration
         if (source != null) {
             return source;
@@ -678,21 +777,21 @@ public abstract class AbstractOsgiCompilerMojo extends AbstractCompilerMojo impl
         if (javacSource != null) {
             return javacSource;
         }
-        // then, execution environment
-        String eeSource = ee.getCompilerSourceLevelDefault();
-        if (eeSource != null) {
-            return eeSource;
+        String profileName = getEclipsePluginProject().getBuildProperties().getJreCompilationProfile();
+        if (profileName != null) {
+            return ExecutionEnvironmentUtils.getExecutionEnvironment(profileName, toolchainManager, session, logger)
+                    .getCompilerSourceLevelDefault();
         }
-
-        return DEFAULT_SOURCE_VERSION;
+        return Arrays.stream(getBREE()) //
+                .map(ExecutionEnvironment::getCompilerSourceLevelDefault) //
+                .filter(Objects::nonNull) //
+                .min(Comparator.comparing(Version::parseVersion)) //
+                .or(() -> Optional.ofNullable(getTargetExecutionEnvironment().getCompilerSourceLevelDefault())) //
+                .orElse(DEFAULT_SOURCE_VERSION);
     }
 
     @Override
     public String getTargetLevel() throws MojoExecutionException {
-        return getTargetLevel(getTargetExecutionEnvironment());
-    }
-
-    public String getTargetLevel(ExecutionEnvironment ee) throws MojoExecutionException {
         // first, explicit POM configuration
         if (target != null) {
             return target;
@@ -702,13 +801,67 @@ public abstract class AbstractOsgiCompilerMojo extends AbstractCompilerMojo impl
         if (javacTarget != null) {
             return javacTarget;
         }
-        // then, execution environment
-        String eeTarget = ee.getCompilerTargetLevelDefault();
-        if (eeTarget != null) {
-            return eeTarget;
+        String profileName = getEclipsePluginProject().getBuildProperties().getJreCompilationProfile();
+        if (profileName != null) {
+            return ExecutionEnvironmentUtils.getExecutionEnvironment(profileName, toolchainManager, session, logger)
+                    .getCompilerTargetLevelDefault();
+        }
+        return Arrays.stream(getBREE()) //
+                .map(ExecutionEnvironment::getCompilerTargetLevelDefault) //
+                .filter(Objects::nonNull) //
+                .min(Comparator.comparing(Version::parseVersion)) //
+                .or(() -> Optional.ofNullable(getTargetExecutionEnvironment().getCompilerTargetLevelDefault())) //
+                .orElse(DEFAULT_TARGET_VERSION);
+    }
+
+    @Override
+    public String getReleaseLevel() throws MojoExecutionException {
+        // first, explicit POM configuration
+        if (release != null) {
+            return release;
         }
 
-        return DEFAULT_TARGET_VERSION;
+        // implicit determination disabled
+        if (!deriveReleaseCompilerArgumentFromTargetLevel) {
+            return null;
+        }
+
+        String targetLevel = getTargetLevel();
+        String[] targetLevelSplit = targetLevel.split("\\.");
+
+        String releaseLevel;
+        if (targetLevelSplit.length == 1 && targetLevelSplit[0].matches("\\d+")) {
+            releaseLevel = targetLevelSplit[0];
+        } else if (targetLevelSplit.length == 2 && "1".equals(targetLevelSplit[0])
+                && targetLevelSplit[1].matches("\\d+")) {
+            releaseLevel = targetLevelSplit[1];
+        } else {
+            logger.debug("Cannot determining 'maven.compiler.release' property automatically, because target level '"
+                    + targetLevel + "' has an unexpected format.");
+            return null;
+        }
+
+        CtSym ctSym;
+        try {
+            ctSym = JRTUtil.getCtSym(Paths.get(System.getProperty("java.home")));
+        } catch (IOException e) {
+            logger.warn("Unable to determine 'maven.compiler.release' property automatically", e);
+            return null;
+        }
+
+        // TODO: Replace this with CtSym#getReleaseCode(String) once eclipse.jdt.core/5ba272a2d4a7478e0eb3951208ab49b7c069f37d is available with newer ECJ release
+        int releaseLevelInt = Integer.parseInt(releaseLevel);
+        String releaseCode = releaseLevelInt < 10 ? releaseLevel
+                : String.valueOf((char) ('A' + (releaseLevelInt - 10)));
+
+        List<Path> releaseRoots = ctSym.releaseRoots(releaseCode);
+        if (!releaseRoots.isEmpty()) {
+            return releaseLevel;
+        } else {
+            logger.debug("Not determining 'maven.compiler.release' property automatically, because level '"
+                    + releaseLevel + "' is not supported by compiler.");
+            return null;
+        }
     }
 
     private void checkTargetLevelCompatibleWithManifestBREEs(String effectiveTargetLevel,
