@@ -18,7 +18,11 @@ import static org.hamcrest.MatcherAssert.assertThat;
 import java.io.File;
 import java.io.IOException;
 import java.util.ArrayList;
+import java.util.Arrays;
+import java.util.Collections;
 import java.util.List;
+import java.util.Set;
+import java.util.stream.Collectors;
 
 import org.apache.bcel.classfile.ClassFormatException;
 import org.apache.bcel.classfile.ClassParser;
@@ -30,14 +34,17 @@ import org.apache.maven.project.MavenProject;
 import org.codehaus.plexus.util.FileUtils;
 import org.eclipse.tycho.classpath.SourcepathEntry;
 import org.eclipse.tycho.compiler.AbstractOsgiCompilerMojo;
+import org.eclipse.tycho.core.ee.StandardExecutionEnvironment;
 import org.eclipse.tycho.testing.AbstractTychoMojoTestCase;
+import org.hamcrest.Matchers;
+import org.hamcrest.core.StringContains;
 
 import copied.org.apache.maven.plugin.CompilationFailureException;
 
 public class OsgiCompilerTest extends AbstractTychoMojoTestCase {
 
     private static final int TARGET_1_4 = 48;
-    private static final int TARGET_9 = 53;
+    private static final int TARGET_1_8 = 52;
 
     protected File storage;
 
@@ -230,21 +237,21 @@ public class OsgiCompilerTest extends AbstractTychoMojoTestCase {
         assertFalse(new File(project.getBasedir(), "target/classes/testresources/test.properties").exists());
     }
 
-    public void testExecutionEnvironment() throws Exception {
+    public void testSourceCompileLevel() throws Exception {
         File basedir = getBasedir("projects/executionEnvironment");
         List<MavenProject> projects = getSortedProjects(basedir, null);
         MavenProject project;
         // project with neither POM nor MANIFEST configuration => must fallback to 
-        // source/target level == 9
+        // source/target level == 11
         project = projects.get(1);
         getMojo(projects, project).execute();
-        assertBytecodeMajorLevel(TARGET_9, new File(project.getBasedir(), "target/classes/Generic.class"));
+        assertBytecodeMajorLevel(55 /* Java 11 */, new File(project.getBasedir(), "target/classes/Generic.class"));
 
         // project with multiple execution envs.
         // Minimum source and target level must be taken
         project = projects.get(2);
         AbstractOsgiCompilerMojo mojo = getMojo(projects, project);
-        assertEquals("OSGi/Minimum-1.0", mojo.getExecutionEnvironment());
+        assertEquals("1.3", mojo.getSourceLevel());
         try {
             mojo.execute();
             fail("compilation failure due to assert keyword expected");
@@ -257,25 +264,61 @@ public class OsgiCompilerTest extends AbstractTychoMojoTestCase {
         mojo = getMojo(projects, project);
         assertEquals("jsr14", mojo.getTargetLevel());
         assertEquals("1.5", mojo.getSourceLevel());
-        assertEquals("J2SE-1.5", mojo.getExecutionEnvironment());
         mojo.execute();
         assertBytecodeMajorLevel(TARGET_1_4, new File(project.getBasedir(), "target/classes/Generic.class"));
         // project with both explicit EE configuration in pom.xml and Bundle-RequiredExecutionEnvironment.
         // explicit configuration in the pom.xml win
         project = projects.get(4);
         mojo = getMojo(projects, project);
-        assertEquals("J2SE-1.5", mojo.getExecutionEnvironment());
+        mojo.execute();
+        assertEquals("1.3", mojo.getSourceLevel());
         // project with both explicit compiler configuration in build.properties and Bundle-RequiredExecutionEnvironment. 
         // build.properties should win. 
         project = projects.get(5);
         mojo = getMojo(projects, project);
+        mojo.execute();
         assertEquals("jsr14", mojo.getTargetLevel());
         assertEquals("1.5", mojo.getSourceLevel());
-        assertEquals("J2SE-1.5", mojo.getExecutionEnvironment());
-        mojo.execute();
         assertBytecodeMajorLevel(TARGET_1_4, new File(project.getBasedir(), "target/classes/Generic.class"));
     }
 
+    public void testNewerEEthanBREE() throws Exception {
+        File basedir = getBasedir("projects/executionEnvironment/p006-newerEEthanBREE");
+        List<MavenProject> projects = getSortedProjects(basedir, null);
+        MavenProject project = projects.get(0);
+        AbstractOsgiCompilerMojo mojo = getMojo(projects, project);
+        mojo.execute();
+        assertTrue(Integer.parseInt(mojo.getExecutionEnvironment().substring("JavaSE-".length())) >= 11);
+        assertEquals("1.8", mojo.getSourceLevel());
+        assertEquals("1.8", mojo.getTargetLevel());
+        assertEquals("8", mojo.getReleaseLevel());
+        assertBytecodeMajorLevel(TARGET_1_8, new File(project.getBasedir(), "target/classes/Noop.class"));
+    }
+
+    public void testNoBREEButEERequirement() throws Exception {
+        File basedir = getBasedir("projects/executionEnvironment/eeAsRequirement");
+        List<MavenProject> projects = getSortedProjects(basedir, null);
+        MavenProject project = projects.get(0);
+        AbstractOsgiCompilerMojo mojo = getMojo(projects, project);
+        StandardExecutionEnvironment[] ees = mojo.getBREE();
+        assertEquals(Set.of("JavaSE-1.8", "JavaSE/compact1-1.8"),
+                Arrays.stream(ees).map(StandardExecutionEnvironment::getProfileName).collect(Collectors.toSet()));
+    }
+
+    public void testAutomaticReleaseCompilerArgumentDeterminationDisabled() throws Exception {
+        File basedir = getBasedir(
+                "projects/executionEnvironment/p007-automaticReleaseCommpilerArgumentDeterminationDisabled");
+        List<MavenProject> projects = getSortedProjects(basedir, null);
+        MavenProject project = projects.get(0);
+        AbstractOsgiCompilerMojo mojo = getMojo(projects, project);
+        mojo.execute();
+        assertTrue(Integer.parseInt(mojo.getExecutionEnvironment().substring("JavaSE-".length())) >= 11);
+        assertEquals("1.8", mojo.getSourceLevel());
+        assertEquals("1.8", mojo.getTargetLevel());
+        assertNull(mojo.getReleaseLevel());
+        assertBytecodeMajorLevel(TARGET_1_8, new File(project.getBasedir(), "target/classes/Noop.class"));
+    }
+
     private void assertBytecodeMajorLevel(int majorLevel, File classFile) throws ClassFormatException, IOException {
         assertTrue(classFile.canRead());
         JavaClass javaClass = new ClassParser(classFile.getAbsolutePath()).parse();
@@ -363,6 +406,9 @@ public class OsgiCompilerTest extends AbstractTychoMojoTestCase {
         List<MavenProject> projects = getSortedProjects(basedir, null);
         MavenProject project = projects.get(0);
         AbstractOsgiCompilerMojo mojo = getMojo(projects, project);
+
+        setVariableValueToObject(mojo, "showWarnings", Boolean.TRUE);
+
         final List<CharSequence> warnings = new ArrayList<>();
         mojo.setLog(new SystemStreamLog() {
 
@@ -393,6 +439,7 @@ public class OsgiCompilerTest extends AbstractTychoMojoTestCase {
         MavenProject project = projects.get(0);
         AbstractOsgiCompilerMojo mojo = getMojo(projects, project);
         setVariableValueToObject(mojo, "useProjectSettings", Boolean.TRUE);
+        setVariableValueToObject(mojo, "showWarnings", Boolean.TRUE);
         final List<CharSequence> warnings = new ArrayList<>();
         mojo.setLog(new SystemStreamLog() {
 
@@ -446,6 +493,10 @@ public class OsgiCompilerTest extends AbstractTychoMojoTestCase {
     }
 
     public void test367431_frameworkExtensionCompileAccessRules() throws Exception {
+        // This test assumes that the internal class com.sun.crypto.provider.SunJCE exists and its package is not exported.
+        // This is the case for all supported JDKs to date (1.8, 11, 14).
+        // Note: The bundle uses BREE 1.8 here, because apparently this kind of framework-extension does not
+        // correctly work with modular API (Java9+).
         File basedir = getBasedir("projects/367431_frameworkExtensionCompileAccessRules/bundle");
         List<MavenProject> projects = getSortedProjects(basedir,
                 new File("src/test/resources/projects/367431_frameworkExtensionCompileAccessRules/repository"));
@@ -527,4 +578,29 @@ public class OsgiCompilerTest extends AbstractTychoMojoTestCase {
                     + " plugin parameter and the custom compiler argument '-log'. Only either of them is allowed."));
         }
     }
+
+    public void testJreCompilationProfile() throws Exception {
+        File basedir = getBasedir("projects/jreCompilationProfile");
+        List<MavenProject> projects = getSortedProjects(basedir, null);
+        MavenProject project = projects.get(0);
+        AbstractOsgiCompilerMojo mojo = getMojo(Collections.singletonList(project), project);
+        mojo.execute();
+        assertEquals("1.8", mojo.getSourceLevel());
+        assertEquals("1.8", mojo.getTargetLevel());
+        assertBytecodeMajorLevel(TARGET_1_8, new File(project.getBasedir(), "target/classes/Test.class"));
+    }
+
+    public void testUseJDKBREE() throws Exception {
+        File basedir = getBasedir("projects/useJDKBREE");
+        List<MavenProject> projects = getSortedProjects(basedir, null);
+        MavenProject project = projects.get(0);
+        AbstractOsgiCompilerMojo mojo = getMojo(Collections.singletonList(project), project);
+        try {
+            mojo.execute();
+            fail("Mojo should fail because of missing toolchains");
+        } catch (MojoExecutionException ex) {
+            assertThat(ex.getMessage(), Matchers.allOf(StringContains.containsStringIgnoringCase("toolchain"),
+                    StringContains.containsString("JavaSE-1.8")));
+        }
+    }
 }
