@@ -13,13 +13,14 @@ package org.eclipse.tycho.core;
 import java.util.List;
 
 import org.apache.maven.project.MavenProject;
+import org.eclipse.tycho.ReactorProject;
 import org.eclipse.tycho.classpath.ClasspathEntry;
 import org.eclipse.tycho.classpath.ClasspathEntry.AccessRule;
 
 public interface BundleProject extends TychoProject {
-    public List<ClasspathEntry> getClasspath(MavenProject project);
+    public List<ClasspathEntry> getClasspath(ReactorProject project);
 
-    public List<AccessRule> getBootClasspathExtraAccessRules(MavenProject project);
+    public List<AccessRule> getBootClasspathExtraAccessRules(ReactorProject project);
 
     /**
      * Returns the value of the specified attribute key in the project's MANIFEST, or null if the
