@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 2008, 2019 Sonatype Inc. and others.
+ * Copyright (c) 2008, 2020 Sonatype Inc. and others.
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
@@ -13,9 +13,9 @@ package org.eclipse.tycho.core.test;
 import java.io.File;
 import java.io.IOException;
 import java.util.ArrayList;
-import java.util.Arrays;
 import java.util.List;
-import java.util.Properties;
+import java.util.Set;
+import java.util.stream.Collectors;
 
 import org.apache.maven.execution.MavenSession;
 import org.apache.maven.model.Dependency;
@@ -27,12 +27,11 @@ import org.eclipse.tycho.artifacts.DependencyArtifacts;
 import org.eclipse.tycho.classpath.ClasspathEntry;
 import org.eclipse.tycho.core.TargetPlatformConfiguration;
 import org.eclipse.tycho.core.TychoProject;
+import org.eclipse.tycho.core.osgitools.DefaultReactorProject;
 import org.eclipse.tycho.core.osgitools.OsgiBundleProject;
 import org.eclipse.tycho.core.resolver.DefaultTargetPlatformConfigurationReader;
 import org.eclipse.tycho.core.resolver.TargetPlatformConfigurationException;
 import org.eclipse.tycho.core.shared.TargetEnvironment;
-import org.eclipse.tycho.core.utils.TychoProjectUtils;
-import org.eclipse.tycho.core.utils.TychoVersion;
 import org.eclipse.tycho.testing.AbstractTychoMojoTestCase;
 
 public class TychoTest extends AbstractTychoMojoTestCase {
@@ -77,7 +76,7 @@ public class TychoTest extends AbstractTychoMojoTestCase {
         } catch (Exception e) {
 //	        List<Exception> exceptions = result.getExceptions();
 //	        assertEquals(1, exceptions.size());
-            assertTrue(e.getMessage().contains("Missing Constraint: Import-Package: moduleorder.p002"));
+            assertTrue(e.getMessage().contains("Unresolved requirement: Import-Package: moduleorder.p002"));
         }
     }
 
@@ -124,40 +123,36 @@ public class TychoTest extends AbstractTychoMojoTestCase {
         for (MavenProject project : projects) {
             artifactIds.add(project.getArtifactId());
         }
-        assertEquals(Arrays.asList("parent", "host", "dep", "fragment", "fragment2", "client"), artifactIds);
+        assertEquals(0, artifactIds.indexOf("parent"));
+        assertEquals(artifactIds.size() - 1, artifactIds.indexOf("client"));
+        assertTrue(artifactIds.indexOf("host") < artifactIds.indexOf("fragment2"));
+        assertTrue(artifactIds.indexOf("host") < artifactIds.indexOf("fragment")
+                && artifactIds.indexOf("dep") < artifactIds.indexOf("fragment"));
 
-        MavenProject host = projects.get(1);
-        MavenProject dep = projects.get(2);
-        MavenProject fragment = projects.get(3);
-        MavenProject fragment2 = projects.get(4);
-        MavenProject client = projects.get(5);
-
-        assertEquals("host", host.getArtifactId());
         // host does not know anything about fragments
+        MavenProject host = projects.stream().filter(p -> p.getArtifactId().equals("host")).findAny().get();
         List<Dependency> hostDependencies = host.getModel().getDependencies();
         assertEquals(0, hostDependencies.size());
 
-        assertEquals("fragment", fragment.getArtifactId());
+        MavenProject fragment = projects.stream().filter(p -> p.getArtifactId().equals("fragment")).findAny().get();
         List<Dependency> fragmentDependencies = fragment.getModel().getDependencies();
         // host first, then fragment dependency
         assertEquals(2, fragmentDependencies.size());
         assertEquals("host", fragmentDependencies.get(0).getArtifactId());
         assertEquals("dep", fragmentDependencies.get(1).getArtifactId());
 
-        assertEquals("fragment2", fragment2.getArtifactId());
         // host only
+        MavenProject fragment2 = projects.stream().filter(p -> p.getArtifactId().equals("fragment2")).findAny().get();
         List<Dependency> fragment2Dependencies = fragment2.getModel().getDependencies();
         assertEquals(1, fragment2Dependencies.size());
         assertEquals("host", fragment2Dependencies.get(0).getArtifactId());
 
-        assertEquals("client", client.getArtifactId());
         // depends on host and because host has ExtensibleAPI also depends fragment and fragent2
+        MavenProject client = projects.stream().filter(p -> p.getArtifactId().equals("client")).findAny().get();
         List<Dependency> clientDependencies = client.getModel().getDependencies();
         assertEquals(4, clientDependencies.size());
-        assertEquals("host", clientDependencies.get(0).getArtifactId());
-        assertEquals("fragment", clientDependencies.get(1).getArtifactId());
-        assertEquals("dep", clientDependencies.get(2).getArtifactId());
-        assertEquals("fragment2", clientDependencies.get(3).getArtifactId());
+        assertEquals(Set.of("host", "fragment", "dep", "fragment2"),
+                clientDependencies.stream().map(Dependency::getArtifactId).collect(Collectors.toSet()));
     }
 
     public void testMNGECLIPSE942() throws Exception {
@@ -166,7 +161,7 @@ public class TychoTest extends AbstractTychoMojoTestCase {
         File platformLocation = new File("src/test/resources/targetplatforms/MNGECLIPSE-942");
         MavenProject project = getSortedProjects(basedir, platformLocation).get(0);
         TychoProject projectType = lookup(TychoProject.class, project.getPackaging());
-        DependencyArtifacts platform = projectType.getDependencyArtifacts(project);
+        DependencyArtifacts platform = projectType.getDependencyArtifacts(DefaultReactorProject.adapt(project));
 
         assertEquals(2, platform.getArtifacts(ArtifactType.TYPE_ECLIPSE_PLUGIN).size());
         assertNotNull(platform.getArtifact(ArtifactType.TYPE_ECLIPSE_PLUGIN, "org.junit4.nl_ru", null));
@@ -180,7 +175,7 @@ public class TychoTest extends AbstractTychoMojoTestCase {
 
         OsgiBundleProject projectType = (OsgiBundleProject) lookup(TychoProject.class, project.getPackaging());
 
-        List<ClasspathEntry> classpath = projectType.getClasspath(project);
+        List<ClasspathEntry> classpath = projectType.getClasspath(DefaultReactorProject.adapt(project));
 
         assertEquals(3, classpath.size());
         assertEquals(1, classpath.get(0).getLocations().size());
@@ -202,12 +197,11 @@ public class TychoTest extends AbstractTychoMojoTestCase {
         List<MavenProject> projects = getSortedProjects(basedir, platformLocation);
         assertEquals(3, projects.size());
 
-        MavenProject b01 = projects.get(1);
         MavenProject b02 = projects.get(2);
 
         OsgiBundleProject projectType = (OsgiBundleProject) lookup(TychoProject.class, b02.getPackaging());
 
-        List<ClasspathEntry> classpath = projectType.getClasspath(b02);
+        List<ClasspathEntry> classpath = projectType.getClasspath(DefaultReactorProject.adapt(b02));
 
         assertEquals(5, classpath.size());
 
@@ -353,32 +347,6 @@ public class TychoTest extends AbstractTychoMojoTestCase {
         }
     }
 
-    public void testBundleRuntimeExecutionEnvironment() throws Exception {
-        File basedir = getBasedir("projects/bree");
-
-        Properties properties = new Properties();
-        properties.put("tycho-version", TychoVersion.getTychoVersion());
-
-        List<MavenProject> projects = getSortedProjects(basedir, properties, null);
-        assertEquals(6, projects.size());
-        int i = 0;
-
-        assertEquals("executionenvironment.manifest", projects.get(++i).getArtifactId());
-        assertEquals("OSGi/Minimum-1.0", getActiveEEProfile(projects.get(i)));
-
-        assertEquals("executionenvironment.manifest-minimal", projects.get(++i).getArtifactId());
-        assertEquals("OSGi/Minimum-1.0", getActiveEEProfile(projects.get(i)));
-
-        assertEquals("executionenvironment.pom-hard", projects.get(++i).getArtifactId());
-        assertEquals("OSGi/Minimum-1.2", getActiveEEProfile(projects.get(i)));
-
-        assertEquals("executionenvironment.buildproperties", projects.get(++i).getArtifactId());
-        assertEquals("OSGi/Minimum-1.1", getActiveEEProfile(projects.get(i)));
-
-        assertEquals("executionenvironment.pom-default", projects.get(++i).getArtifactId());
-        assertEquals("OSGi/Minimum-1.2", getActiveEEProfile(projects.get(i)));
-    }
-
     public void testWithProjectReferencesItself() throws Exception {
         File basedir = getBasedir("projects/referencesItself");
         try {
@@ -390,7 +358,4 @@ public class TychoTest extends AbstractTychoMojoTestCase {
 
     }
 
-    private static String getActiveEEProfile(MavenProject project) {
-        return TychoProjectUtils.getExecutionEnvironmentConfiguration(project).getProfileName();
-    }
 }
