@@ -1,16 +1,19 @@
 /*******************************************************************************
- * Copyright (c) 2015 VDS Rail and others.
- * All rights reserved. This program and the accompanying materials
- * are made available under the terms of the Eclipse Public License v1.0
+ * Copyright (c) 2015, 2020 VDS Rail and others.
+ * This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License 2.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/epl-v10.html
+ * https://www.eclipse.org/legal/epl-2.0/
+ *
+ * SPDX-License-Identifier: EPL-2.0
  *
  * Contributors:
  *    Enrico De Fent - initial API and implementation
  *******************************************************************************/
 package org.eclipse.tycho.extras.docbundle;
 
-import org.junit.Assert;
+import static org.junit.jupiter.api.Assertions.assertEquals;
+import static org.junit.jupiter.api.Assertions.fail;
 
 class AssertOnBuffer {
     final String[] lines;
@@ -20,25 +23,25 @@ class AssertOnBuffer {
     public AssertOnBuffer(String buffer) {
         this.buffer = buffer;
 
-        String lineSeparator = System.getProperty("line.separator");
+        String lineSeparator = System.lineSeparator();
         this.lines = buffer.split(lineSeparator);
     }
 
     public void assertMoreLines() {
         if (curLine == lines.length) {
-            Assert.fail("expected more lines, found no more");
+            fail("expected more lines, found no more");
         }
     }
 
     public void assertNextLine(String line) {
         assertMoreLines();
-        Assert.assertEquals("at line " + curLine, line, lines[curLine]);
+        assertEquals(line, lines[curLine], "at line " + curLine);
         curLine++;
     }
 
     public void assertNoMoreLines() {
         if (curLine < lines.length) {
-            Assert.fail(String.format("expected no more lines, found %d more", lines.length - curLine));
+            fail(String.format("expected no more lines, found %d more", lines.length - curLine));
         }
     }
 }
