@@ -1,9 +1,11 @@
 /*******************************************************************************
  * Copyright (c) 2011, 2018 Sonatype Inc. and others.
- * All rights reserved. This program and the accompanying materials
- * are made available under the terms of the Eclipse Public License v1.0
+ * This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License 2.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/epl-v10.html
+ * https://www.eclipse.org/legal/epl-2.0/
+ *
+ * SPDX-License-Identifier: EPL-2.0
  *
  * Contributors:
  *    Sonatype Inc. - initial API and implementation
@@ -46,6 +48,7 @@ import org.eclipse.sisu.equinox.EquinoxServiceFactory;
 import org.eclipse.tycho.PackagingType;
 import org.eclipse.tycho.artifacts.TargetPlatform;
 import org.eclipse.tycho.core.osgitools.DebugUtils;
+import org.eclipse.tycho.core.osgitools.DefaultReactorProject;
 import org.eclipse.tycho.core.shared.BuildProperties;
 import org.eclipse.tycho.core.shared.BuildPropertiesParser;
 import org.eclipse.tycho.core.utils.TychoProjectUtils;
@@ -84,10 +87,18 @@ import de.pdark.decentxml.Element;
  * values in this file override values of respective keys in
  * <code>&lt;originalFeature&gt;/feature.properties</code>.
  * 
+ * @deprecated this mojo is replaced by the tycho-source-plugin with execution feature-source which
+ *             offers equivalent and even enhanced functionality
  */
-@Mojo(name = "source-feature", defaultPhase = LifecyclePhase.PACKAGE)
+@Mojo(name = "source-feature", defaultPhase = LifecyclePhase.PACKAGE, threadSafe = true)
+@Deprecated(since = "2.2", forRemoval = true)
 public class SourceFeatureMojo extends AbstractMojo {
 
+    /**
+     * Lock object to ensure thread-safety
+     */
+    private static final Object LOCK = new Object();
+
     public static final String FEATURE_TEMPLATE_DIR = "sourceTemplateFeature";
 
     public static final String SOURCES_FEATURE_CLASSIFIER = "sources-feature";
@@ -123,7 +134,7 @@ public class SourceFeatureMojo extends AbstractMojo {
 
     /**
      * Use this to explicitly set the <a href=
-     * "http://help.eclipse.org/juno/index.jsp?topic=%2Forg.eclipse.pde.doc.user%2Fguide%2Ftools%2Feditors%2Ffeature_editor%2Ffeature_editor.htm"
+     * "https://help.eclipse.org/juno/index.jsp?topic=%2Forg.eclipse.pde.doc.user%2Fguide%2Ftools%2Feditors%2Ffeature_editor%2Ffeature_editor.htm"
      * > branding plugin attribute</a> of the generated source feature (overrides
      * {@link #reuseBrandingPlugin}).
      */
@@ -206,45 +217,49 @@ public class SourceFeatureMojo extends AbstractMojo {
 
     @Override
     public void execute() throws MojoExecutionException, MojoFailureException {
+        logger.warn(
+                "Mojo tycho-source-feature-plugin:source-feature is replaced by the tycho-source-plugin:feature-source which offers equivalent and even enhanced functionality");
         if (!PackagingType.TYPE_ECLIPSE_FEATURE.equals(project.getPackaging()) || skip) {
             return;
         }
-        try {
-            Properties sourceFeatureTemplateProps = readSourceTemplateFeatureProperties();
-            Properties mergedSourceFeatureProps = mergeFeatureProperties(sourceFeatureTemplateProps);
-            File sourceFeatureXml = generateSourceFeatureXml(mergedSourceFeatureProps, sourceFeatureTemplateProps);
-            writeProperties(mergedSourceFeatureProps, getMergedSourceFeaturePropertiesFile());
-            MavenArchiver archiver = new MavenArchiver();
-            archiver.setArchiver(jarArchiver);
-            File outputJarFile = getOutputJarFile();
-            archiver.setOutputFile(outputJarFile);
-            File template = new File(project.getBasedir(), FEATURE_TEMPLATE_DIR);
-            if (template.isDirectory()) {
-                DefaultFileSet templateFileSet = new DefaultFileSet();
-                templateFileSet.setDirectory(template);
-                // make sure we use generated feature.xml and feature.properties 
-                templateFileSet.setExcludes(new String[] { Feature.FEATURE_XML, FEATURE_PROPERTIES });
-                archiver.getArchiver().addFileSet(templateFileSet);
+        synchronized (LOCK) {
+            try {
+                Properties sourceFeatureTemplateProps = readSourceTemplateFeatureProperties();
+                Properties mergedSourceFeatureProps = mergeFeatureProperties(sourceFeatureTemplateProps);
+                File sourceFeatureXml = generateSourceFeatureXml(mergedSourceFeatureProps, sourceFeatureTemplateProps);
+                writeProperties(mergedSourceFeatureProps, getMergedSourceFeaturePropertiesFile());
+                MavenArchiver archiver = new MavenArchiver();
+                archiver.setArchiver(jarArchiver);
+                File outputJarFile = getOutputJarFile();
+                archiver.setOutputFile(outputJarFile);
+                File template = new File(project.getBasedir(), FEATURE_TEMPLATE_DIR);
+                if (template.isDirectory()) {
+                    DefaultFileSet templateFileSet = new DefaultFileSet();
+                    templateFileSet.setDirectory(template);
+                    // make sure we use generated feature.xml and feature.properties
+                    templateFileSet.setExcludes(new String[] { Feature.FEATURE_XML, FEATURE_PROPERTIES });
+                    archiver.getArchiver().addFileSet(templateFileSet);
+                }
+
+                BuildProperties buildProperties = buildPropertiesParser.parse(project.getBasedir());
+                archiver.getArchiver().addFileSet(getManuallyIncludedFiles(project.getBasedir(), buildProperties));
+
+                archiver.getArchiver().addFile(sourceFeatureXml, Feature.FEATURE_XML);
+                archiver.getArchiver().addFile(getMergedSourceFeaturePropertiesFile(), FEATURE_PROPERTIES);
+                File licenseFeature = licenseFeatureHelper
+                        .getLicenseFeature(Feature.read(new File(project.getBasedir(), "feature.xml")), project);
+                if (licenseFeature != null) {
+                    archiver.getArchiver()
+                            .addArchivedFileSet(licenseFeatureHelper.getLicenseFeatureFileSet(licenseFeature));
+                }
+                archiver.createArchive(session, project, archive);
+
+                projectHelper.attachArtifact(project, outputJarFile, SOURCES_FEATURE_CLASSIFIER);
+            } catch (MojoExecutionException e) {
+                throw e;
+            } catch (Exception e) {
+                throw new MojoExecutionException("Could not package source feature jar", e);
             }
-
-            BuildProperties buildProperties = buildPropertiesParser.parse(project.getBasedir());
-            archiver.getArchiver().addFileSet(getManuallyIncludedFiles(project.getBasedir(), buildProperties));
-
-            archiver.getArchiver().addFile(sourceFeatureXml, Feature.FEATURE_XML);
-            archiver.getArchiver().addFile(getMergedSourceFeaturePropertiesFile(), FEATURE_PROPERTIES);
-            File licenseFeature = licenseFeatureHelper
-                    .getLicenseFeature(Feature.read(new File(project.getBasedir(), "feature.xml")), project);
-            if (licenseFeature != null) {
-                archiver.getArchiver()
-                        .addArchivedFileSet(licenseFeatureHelper.getLicenseFeatureFileSet(licenseFeature));
-            }
-            archiver.createArchive(session, project, archive);
-
-            projectHelper.attachArtifact(project, outputJarFile, SOURCES_FEATURE_CLASSIFIER);
-        } catch (MojoExecutionException e) {
-            throw e;
-        } catch (Exception e) {
-            throw new MojoExecutionException("Could not package source feature jar", e);
         }
     }
 
@@ -276,7 +291,8 @@ public class SourceFeatureMojo extends AbstractMojo {
 
         final Feature sourceFeature = createSourceFeatureSkeleton(feature, mergedSourceFeatureProps,
                 sourceTemplateProps);
-        fillReferences(sourceFeature, feature, TychoProjectUtils.getTargetPlatform(project));
+        fillReferences(sourceFeature, feature,
+                TychoProjectUtils.getTargetPlatform(DefaultReactorProject.adapt(project)));
 
         Feature.write(sourceFeature, sourceFeatureXml, "  ");
         return sourceFeatureXml;
