@@ -1,9 +1,11 @@
 /*******************************************************************************
  * Copyright (c) 2008, 2011 Sonatype Inc. and others.
- * All rights reserved. This program and the accompanying materials
- * are made available under the terms of the Eclipse Public License v1.0
+ * This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License 2.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/epl-v10.html
+ * https://www.eclipse.org/legal/epl-2.0/
+ *
+ * SPDX-License-Identifier: EPL-2.0
  *
  * Contributors:
  *    Sonatype Inc. - initial API and implementation
@@ -18,24 +20,17 @@ import org.eclipse.osgi.service.resolver.State;
 import org.osgi.framework.Bundle;
 import org.osgi.framework.ServiceReference;
 
-public class Eclipse35Test
-    extends TestCase
-{
-    public void test()
-        throws Exception
-    {
-        Bundle equinox = getBundle( "org.eclipse.osgi");
+public class Eclipse35Test extends TestCase {
+    public void test() throws Exception {
+        Bundle equinox = getBundle("org.eclipse.osgi");
 
-        assertEquals( 3, equinox.getVersion().getMajor() );
-        assertEquals( 15, equinox.getVersion().getMinor() );
+        assertEquals(3, equinox.getVersion().getMajor());
+        assertEquals(16, equinox.getVersion().getMinor());
     }
-    
-    public Bundle getBundle( String id )
-    {
-        for ( Bundle bundle : Activator.context.getBundles() )
-        {
-            if ( id.equals( bundle.getSymbolicName() ) )
-            {
+
+    public Bundle getBundle(String id) {
+        for (Bundle bundle : Activator.context.getBundles()) {
+            if (id.equals(bundle.getSymbolicName())) {
                 return bundle;
             }
         }
