@@ -1,9 +1,11 @@
 /*******************************************************************************
- * Copyright (c) 2012, 2018 Sonatype Inc. and others.
- * All rights reserved. This program and the accompanying materials
- * are made available under the terms of the Eclipse Public License v1.0
+ * Copyright (c) 2012, 2020 Sonatype Inc. and others.
+ * This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License 2.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/epl-v10.html
+ * https://www.eclipse.org/legal/epl-2.0/
+ *
+ * SPDX-License-Identifier: EPL-2.0
  *
  * Contributors:
  *    Sonatype Inc. - initial API and implementation
@@ -21,7 +23,6 @@ import java.util.zip.ZipException;
 import java.util.zip.ZipFile;
 
 import org.apache.maven.it.Verifier;
-import org.codehaus.plexus.util.IOUtil;
 import org.eclipse.tycho.model.Feature;
 import org.eclipse.tycho.test.AbstractTychoIntegrationTest;
 import org.eclipse.tycho.test.util.ResourceUtil;
@@ -57,11 +58,9 @@ public class LicenseFeatureTest extends AbstractTychoIntegrationTest {
             Assert.assertNotNull(zip.getEntry("file2.txt"));
 
             Properties p = new Properties();
-            InputStream is = zip.getInputStream(zip.getEntry("feature.properties"));
-            try {
+
+            try (InputStream is = zip.getInputStream(zip.getEntry("feature.properties"))) {
                 p.load(is);
-            } finally {
-                IOUtil.close(is);
             }
 
             Feature featureXML = Feature.readJar(feature);
