@@ -1,9 +1,11 @@
 /*******************************************************************************
- * Copyright (c) 2019 Guillaume Dufour and others.
- * All rights reserved. This program and the accompanying materials
- * are made available under the terms of the Eclipse Public License v1.0
+ * Copyright (c) 2019, 2021 Guillaume Dufour and others.
+ * This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License 2.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/epl-v10.html
+ * https://www.eclipse.org/legal/epl-2.0/
+ *
+ * SPDX-License-Identifier: EPL-2.0
  *
  * Contributors:
  *    Guillaume Dufour - support repo ref location (453708)
@@ -39,7 +41,7 @@ public class RepoRefLocationP2RepositoryIntegrationTest extends AbstractTychoInt
     public static void executeBuild() throws Exception {
         verifier = new RepoRefLocationP2RepositoryIntegrationTest().getVerifier("/p2Repository.repositoryRef.location",
                 false);
-        verifier.getCliOptions().add("-Dtest-data-repo=" + ResourceUtil.P2Repositories.ECLIPSE_OXYGEN.toString());
+        verifier.getCliOptions().add("-Dtest-data-repo=" + ResourceUtil.P2Repositories.ECLIPSE_LATEST.toString());
         verifier.executeGoal("package");
         verifier.verifyErrorFreeLog();
         p2Repo = P2RepositoryTool.forEclipseRepositoryModule(new File(verifier.getBasedir()));
