@@ -1,13 +1,16 @@
 /*******************************************************************************
  * Copyright (c) 2008, 2019 Sonatype Inc. and others.
- * All rights reserved. This program and the accompanying materials
- * are made available under the terms of the Eclipse Public License v1.0
+ * This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License 2.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/epl-v10.html
+ * https://www.eclipse.org/legal/epl-2.0/
+ *
+ * SPDX-License-Identifier: EPL-2.0
  *
  * Contributors:
  *    Sonatype Inc. - initial API and implementation
- *    Christoph Läubrich - fix Bug 551739
+ *    Christoph Läubrich    - Bug 551739, Bug 538144, Bug 533747
+ *                          - [Bug 567098] pomDependencies=consider should wrap non-osgi jars
  *******************************************************************************/
 package org.eclipse.tycho.p2.resolver;
 
@@ -60,12 +63,13 @@ import org.eclipse.tycho.artifacts.TargetPlatform;
 import org.eclipse.tycho.core.DependencyResolver;
 import org.eclipse.tycho.core.DependencyResolverConfiguration;
 import org.eclipse.tycho.core.TargetPlatformConfiguration;
+import org.eclipse.tycho.core.TargetPlatformConfiguration.PomDependencies;
 import org.eclipse.tycho.core.TychoConstants;
 import org.eclipse.tycho.core.TychoProject;
+import org.eclipse.tycho.core.ee.TargetDefinitionFile;
 import org.eclipse.tycho.core.ee.shared.ExecutionEnvironmentConfiguration;
 import org.eclipse.tycho.core.maven.MavenDependencyInjector;
 import org.eclipse.tycho.core.maven.utils.PluginRealmHelper;
-import org.eclipse.tycho.core.maven.utils.PluginRealmHelper.PluginFilter;
 import org.eclipse.tycho.core.osgitools.AbstractTychoProject;
 import org.eclipse.tycho.core.osgitools.BundleReader;
 import org.eclipse.tycho.core.osgitools.DebugUtils;
@@ -73,7 +77,6 @@ import org.eclipse.tycho.core.osgitools.DefaultReactorProject;
 import org.eclipse.tycho.core.osgitools.targetplatform.DefaultDependencyArtifacts;
 import org.eclipse.tycho.core.osgitools.targetplatform.MultiEnvironmentDependencyArtifacts;
 import org.eclipse.tycho.core.p2.P2ArtifactRepositoryLayout;
-import org.eclipse.tycho.core.resolver.shared.IncludeSourceMode;
 import org.eclipse.tycho.core.resolver.shared.MavenRepositoryLocation;
 import org.eclipse.tycho.core.resolver.shared.OptionalResolutionAction;
 import org.eclipse.tycho.core.shared.BuildFailureException;
@@ -127,7 +130,7 @@ public class P2DependencyResolver extends AbstractLogEnabled implements Dependen
     @Override
     public void setupProjects(final MavenSession session, final MavenProject project,
             final ReactorProject reactorProject) {
-        TargetPlatformConfiguration configuration = (TargetPlatformConfiguration) project
+        TargetPlatformConfiguration configuration = (TargetPlatformConfiguration) reactorProject
                 .getContextValue(TychoConstants.CTX_TARGET_PLATFORM_CONFIGURATION);
         List<TargetEnvironment> environments = configuration.getEnvironments();
         Map<String, IDependencyMetadata> metadata = getDependencyMetadata(session, project, environments,
@@ -152,27 +155,19 @@ public class P2DependencyResolver extends AbstractLogEnabled implements Dependen
 
         // let external providers contribute additional metadata
         try {
-            pluginRealmHelper.execute(session, project, new Runnable() {
-                @Override
-                public void run() {
-                    try {
-                        for (P2MetadataProvider provider : plexus.lookupList(P2MetadataProvider.class)) {
-                            Map<String, IDependencyMetadata> providedMetadata = provider.getDependencyMetadata(session,
-                                    project, null, optionalAction);
-                            if (providedMetadata != null) {
-                                metadata.putAll(providedMetadata);
-                            }
+            pluginRealmHelper.execute(session, project, () -> {
+                try {
+                    for (P2MetadataProvider provider : plexus.lookupList(P2MetadataProvider.class)) {
+                        Map<String, IDependencyMetadata> providedMetadata = provider.getDependencyMetadata(session,
+                                project, null, optionalAction);
+                        if (providedMetadata != null) {
+                            metadata.putAll(providedMetadata);
                         }
-                    } catch (ComponentLookupException e) {
-                        // have not found anything
                     }
+                } catch (ComponentLookupException e) {
+                    // have not found anything
                 }
-            }, new PluginFilter() {
-                @Override
-                public boolean accept(PluginDescriptor descriptor) {
-                    return isTychoP2Plugin(descriptor);
-                }
-            });
+            }, this::isTychoP2Plugin);
         } catch (MavenExecutionException e) {
             throw new RuntimeException(e);
         }
@@ -196,35 +191,29 @@ public class P2DependencyResolver extends AbstractLogEnabled implements Dependen
     @Override
     public TargetPlatform computePreliminaryTargetPlatform(MavenSession session, MavenProject project,
             List<ReactorProject> reactorProjects) {
-        TargetPlatformConfiguration configuration = TychoProjectUtils.getTargetPlatformConfiguration(project);
-        ExecutionEnvironmentConfiguration ee = TychoProjectUtils.getExecutionEnvironmentConfiguration(project);
+        ReactorProject reactorProject = DefaultReactorProject.adapt(project);
+        TargetPlatformConfiguration configuration = TychoProjectUtils.getTargetPlatformConfiguration(reactorProject);
+        ExecutionEnvironmentConfiguration ee = TychoProjectUtils.getExecutionEnvironmentConfiguration(reactorProject);
 
         TargetPlatformConfigurationStub tpConfiguration = new TargetPlatformConfigurationStub();
         tpConfiguration.setIncludePackedArtifacts(configuration.isIncludePackedArtifacts());
 
-        PomDependencyCollector pomDependencies = null;
-        if (TargetPlatformConfiguration.POM_DEPENDENCIES_CONSIDER.equals(configuration.getPomDependencies())) {
-            pomDependencies = collectPomDependencies(project, reactorProjects, session);
-        } else {
-            // TODO 412416 remove this when the setProjectLocation is no longer needed
-            pomDependencies = resolverFactory.newPomDependencyCollector();
-            pomDependencies.setProjectLocation(project.getBasedir());
-        }
-
+        PomDependencyCollector pomDependencies = collectPomDependencies(project, reactorProjects, session,
+                configuration.getPomDependencies());
         for (ArtifactRepository repository : project.getRemoteArtifactRepositories()) {
             addEntireP2RepositoryToTargetPlatform(repository, tpConfiguration);
         }
 
         tpConfiguration.setEnvironments(configuration.getEnvironments());
         for (File file : configuration.getTargets()) {
-            addTargetFileContentToTargetPlatform(file, configuration.getTargetDefinitionIncludeSourceMode(),
-                    tpConfiguration);
+            addTargetFileContentToTargetPlatform(file, tpConfiguration);
         }
 
         tpConfiguration.addFilters(configuration.getFilters());
+        tpConfiguration.setIncludeSourceMode(configuration.getTargetDefinitionIncludeSourceMode());
 
-        return reactorRepositoryManager.computePreliminaryTargetPlatform(DefaultReactorProject.adapt(project),
-                tpConfiguration, ee, reactorProjects, pomDependencies);
+        return reactorRepositoryManager.computePreliminaryTargetPlatform(reactorProject, tpConfiguration, ee,
+                reactorProjects, pomDependencies);
     }
 
     private ReactorProject getThisReactorProject(MavenSession session, MavenProject project,
@@ -252,7 +241,11 @@ public class P2DependencyResolver extends AbstractLogEnabled implements Dependen
     }
 
     private PomDependencyCollector collectPomDependencies(MavenProject project, List<ReactorProject> reactorProjects,
-            MavenSession session) {
+            MavenSession session, PomDependencies pomDependencies) {
+        if (pomDependencies == PomDependencies.ignore) {
+            return resolverFactory.newPomDependencyCollector(DefaultReactorProject.adapt(project));
+        }
+
         Set<String> projectIds = new HashSet<>();
         for (ReactorProject p : reactorProjects) {
             String key = ArtifactUtils.key(p.getGroupId(), p.getArtifactId(), p.getVersion());
@@ -295,7 +288,8 @@ public class P2DependencyResolver extends AbstractLogEnabled implements Dependen
         }
         PomDependencyProcessor pomDependencyProcessor = new PomDependencyProcessor(session, repositorySystem,
                 resolverFactory, equinox.getService(LocalRepositoryP2Indices.class), getLogger());
-        return pomDependencyProcessor.collectPomDependencies(project, externalArtifacts);
+        return pomDependencyProcessor.collectPomDependencies(project, externalArtifacts,
+                pomDependencies == PomDependencies.wrapAsBundle);
     }
 
     private void addEntireP2RepositoryToTargetPlatform(ArtifactRepository repository,
@@ -312,9 +306,9 @@ public class P2DependencyResolver extends AbstractLogEnabled implements Dependen
         }
     }
 
-    private void addTargetFileContentToTargetPlatform(File targetFile, IncludeSourceMode includeSourcesMode,
+    private void addTargetFileContentToTargetPlatform(File targetFile,
             TargetPlatformConfigurationStub resolutionContext) {
-        TargetDefinitionFile target = TargetDefinitionFile.read(targetFile, includeSourcesMode);
+        TargetDefinitionFile target = TargetDefinitionFile.read(targetFile);
         resolutionContext.addTargetDefinition(target);
     }
 
@@ -322,13 +316,14 @@ public class P2DependencyResolver extends AbstractLogEnabled implements Dependen
     public DependencyArtifacts resolveDependencies(final MavenSession session, final MavenProject project,
             TargetPlatform targetPlatform, List<ReactorProject> reactorProjects,
             DependencyResolverConfiguration resolverConfiguration) {
+        ReactorProject reactorProject = DefaultReactorProject.adapt(project);
         if (targetPlatform == null) {
-            targetPlatform = TychoProjectUtils.getTargetPlatform(project);
+            targetPlatform = TychoProjectUtils.getTargetPlatform(reactorProject);
         }
 
         // TODO 364134 For compatibility reasons, target-platform-configuration includes settings for the dependency resolution
         // --> split this information logically, e.g. through two distinct interfaces
-        TargetPlatformConfiguration configuration = TychoProjectUtils.getTargetPlatformConfiguration(project);
+        TargetPlatformConfiguration configuration = TychoProjectUtils.getTargetPlatformConfiguration(reactorProject);
 
         P2Resolver osgiResolverImpl = resolverFactory
                 .createResolver(new MavenLoggerAdapter(getLogger(), DebugUtils.isDebugEnabled(session, project)));
@@ -413,11 +408,15 @@ public class P2DependencyResolver extends AbstractLogEnabled implements Dependen
 
         for (P2ResolutionResult.Entry entry : result.getArtifacts()) {
             ArtifactKey key = new DefaultArtifactKey(entry.getType(), entry.getId(), entry.getVersion());
-            ReactorProject otherProject = projects.get(entry.getLocation());
+            ReactorProject otherProject = null;
+            File location = entry.getLocation(false);
+            if (location != null) {
+                otherProject = projects.get(location);
+            }
             if (otherProject != null) {
                 platform.addReactorArtifact(key, otherProject, entry.getClassifier(), entry.getInstallableUnits());
             } else {
-                platform.addArtifactFile(key, entry.getLocation(), entry.getInstallableUnits());
+                platform.addArtifactFile(key, () -> entry.getLocation(true), entry.getInstallableUnits());
             }
         }
         return platform;
