@@ -1,9 +1,11 @@
 /*******************************************************************************
  * Copyright (c) 2008, 2013 Sonatype Inc. and others.
- * All rights reserved. This program and the accompanying materials
- * are made available under the terms of the Eclipse Public License v1.0
+ * This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License 2.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/epl-v10.html
+ * https://www.eclipse.org/legal/epl-2.0/
+ *
+ * SPDX-License-Identifier: EPL-2.0
  *
  * Contributors:
  *    Sonatype Inc. - initial API and implementation
@@ -54,8 +56,9 @@ import org.eclipse.tycho.p2.metadata.IP2Artifact;
 import org.eclipse.tycho.p2.metadata.P2Generator;
 import org.eclipse.tycho.p2.metadata.PublisherOptions;
 
-@Mojo(name = "p2-metadata")
+@Mojo(name = "p2-metadata", threadSafe = true)
 public class P2MetadataMojo extends AbstractMojo {
+    private static final Object LOCK = new Object();
 
     @Parameter(property = "project")
     protected MavenProject project;
@@ -132,7 +135,9 @@ public class P2MetadataMojo extends AbstractMojo {
 
     @Override
     public void execute() throws MojoExecutionException, MojoFailureException {
-        attachP2Metadata();
+        synchronized (LOCK) {
+            attachP2Metadata();
+        }
     }
 
     private <T> T getService(Class<T> type) {
