@@ -1,9 +1,11 @@
 /*******************************************************************************
- * Copyright (c) 2008, 2019 Sonatype Inc. and others.
- * All rights reserved. This program and the accompanying materials
- * are made available under the terms of the Eclipse Public License v1.0
+ * Copyright (c) 2008, 2021 Sonatype Inc. and others.
+ * This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License 2.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/epl-v10.html
+ * https://www.eclipse.org/legal/epl-2.0/
+ *
+ * SPDX-License-Identifier: EPL-2.0
  *
  * Contributors:
  *    Sonatype Inc. - initial API and implementation
@@ -14,7 +16,6 @@ import java.io.BufferedInputStream;
 import java.io.BufferedOutputStream;
 import java.io.File;
 import java.io.FileInputStream;
-import java.io.FileNotFoundException;
 import java.io.FileOutputStream;
 import java.io.IOException;
 import java.io.InputStream;
@@ -56,6 +57,7 @@ import org.eclipse.tycho.core.ArtifactDependencyVisitor;
 import org.eclipse.tycho.core.ArtifactDependencyWalker;
 import org.eclipse.tycho.core.PluginDescription;
 import org.eclipse.tycho.core.osgitools.BundleReader;
+import org.eclipse.tycho.core.osgitools.DefaultReactorProject;
 import org.eclipse.tycho.core.resolver.shared.PlatformPropertiesUtils;
 import org.eclipse.tycho.core.shared.TargetEnvironment;
 import org.eclipse.tycho.core.utils.TychoProjectUtils;
@@ -64,8 +66,9 @@ import org.eclipse.tycho.locking.facade.FileLocker;
 import org.eclipse.tycho.model.BundleConfiguration;
 import org.eclipse.tycho.model.ProductConfiguration;
 
-@Mojo(name = "product-export")
+@Mojo(name = "product-export", threadSafe = true)
 public class ProductExportMojo extends AbstractTychoPackagingMojo {
+    private static final Object LOCK = new Object();
     /**
      * The product configuration, a .product file. This file manages all aspects of a product
      * definition from its constituent plug-ins to configuration files to branding.
@@ -121,101 +124,106 @@ public class ProductExportMojo extends AbstractTychoPackagingMojo {
 
     @Override
     public void execute() throws MojoExecutionException, MojoFailureException {
-        getLog().warn("The packaging type 'eclipse-application' is deprecated; use 'eclipse-repository' instead. "
-                + "See http://wiki.eclipse.org/Tycho_Messages_Explained#Eclipse_Application");
-        if (!productConfigurationFile.exists()) {
-            throw new MojoExecutionException(
-                    "Product configuration file not found " + productConfigurationFile.getAbsolutePath());
-        }
+        synchronized (LOCK) {
+            getLog().warn("The packaging type 'eclipse-application' is deprecated; use 'eclipse-repository' instead. "
+                    + "See https://wiki.eclipse.org/Tycho_Messages_Explained#Eclipse_Application");
+            if (!productConfigurationFile.exists()) {
+                throw new MojoExecutionException(
+                        "Product configuration file not found " + productConfigurationFile.getAbsolutePath());
+            }
 
-        try {
-            getLog().debug("Parsing productConfiguration");
-            productConfiguration = ProductConfiguration.read(productConfigurationFile);
-        } catch (IOException e) {
-            throw new MojoExecutionException("Error reading product configuration file", e);
-        }
+            try {
+                getLog().debug("Parsing productConfiguration");
+                productConfiguration = ProductConfiguration.read(productConfigurationFile);
+            } catch (IOException e) {
+                throw new MojoExecutionException("Error reading product configuration file", e);
+            }
 
-        // build results will vary from system to system without explicit target environment configuration
-        boolean implicitTargetEnvironment = TychoProjectUtils.getTargetPlatformConfiguration(project)
-                .isImplicitTargetEnvironment();
-        if (productConfiguration.includeLaunchers() && implicitTargetEnvironment && environments == null) {
-            throw new MojoFailureException("Product includes native launcher but no target environment was specified");
-        }
+            // build results will vary from system to system without explicit target environment configuration
+            boolean implicitTargetEnvironment = TychoProjectUtils
+                    .getTargetPlatformConfiguration(DefaultReactorProject.adapt(project)).isImplicitTargetEnvironment();
+            if (productConfiguration.includeLaunchers() && implicitTargetEnvironment && environments == null) {
+                throw new MojoFailureException(
+                        "Product includes native launcher but no target environment was specified");
+            }
 
-        if (separateEnvironments) {
-            for (TargetEnvironment environment : getEnvironments()) {
-                File target = getTarget(environment);
+            if (separateEnvironments) {
+                for (TargetEnvironment environment : getEnvironments()) {
+                    File target = getTarget(environment);
+                    File targetEclipse = new File(target, "eclipse");
+                    targetEclipse.mkdirs();
+
+                    generateDotEclipseProduct(targetEclipse);
+                    generateConfigIni(environment, targetEclipse);
+                    includeRootFiles(environment, targetEclipse);
+
+                    ProductAssembler assembler = new ProductAssembler(plexus, manifestReader, targetEclipse,
+                            environment);
+                    assembler.setIncludeSources(includeSources);
+                    getDependencyWalker(environment).walk(assembler);
+
+                    if (productConfiguration.includeLaunchers()) {
+                        copyExecutable(environment, targetEclipse);
+                    }
+
+                    if (createProductArchive) {
+                        createProductArchive(target, toString(environment));
+                    }
+                }
+            } else {
+                File target = getTarget(null);
                 File targetEclipse = new File(target, "eclipse");
                 targetEclipse.mkdirs();
 
                 generateDotEclipseProduct(targetEclipse);
-                generateConfigIni(environment, targetEclipse);
-                includeRootFiles(environment, targetEclipse);
+                generateConfigIni(null, targetEclipse);
 
-                ProductAssembler assembler = new ProductAssembler(session, manifestReader, targetEclipse, environment);
+                for (TargetEnvironment environment : getEnvironments()) {
+                    includeRootFiles(environment, targetEclipse);
+                }
+
+                ProductAssembler assembler = new ProductAssembler(plexus, manifestReader, targetEclipse, null);
                 assembler.setIncludeSources(includeSources);
-                getDependencyWalker(environment).walk(assembler);
+                if (forcePackedDependencies) {
+                    assembler.setUnpackFeatures(false);
+                    assembler.setUnpackPlugins(false);
+                }
+                getDependencyWalker().walk(assembler);
 
                 if (productConfiguration.includeLaunchers()) {
-                    copyExecutable(environment, targetEclipse);
+                    for (TargetEnvironment environment : getEnvironments()) {
+                        copyExecutable(environment, targetEclipse);
+                    }
                 }
 
                 if (createProductArchive) {
-                    createProductArchive(target, toString(environment));
-                }
-            }
-        } else {
-            File target = getTarget(null);
-            File targetEclipse = new File(target, "eclipse");
-            targetEclipse.mkdirs();
-
-            generateDotEclipseProduct(targetEclipse);
-            generateConfigIni(null, targetEclipse);
-
-            for (TargetEnvironment environment : getEnvironments()) {
-                includeRootFiles(environment, targetEclipse);
-            }
-
-            ProductAssembler assembler = new ProductAssembler(session, manifestReader, targetEclipse, null);
-            assembler.setIncludeSources(includeSources);
-            if (forcePackedDependencies) {
-                assembler.setUnpackFeatures(false);
-                assembler.setUnpackPlugins(false);
-            }
-            getDependencyWalker().walk(assembler);
-
-            if (productConfiguration.includeLaunchers()) {
-                for (TargetEnvironment environment : getEnvironments()) {
-                    copyExecutable(environment, targetEclipse);
+                    createProductArchive(target, null);
                 }
             }
 
-            if (createProductArchive) {
-                createProductArchive(target, null);
+            // String version = getTychoProjectFacet().getArtifactKey( project ).getVersion();
+            // String productVersion = VersioningHelper.getExpandedVersion( project, version );
+            // productConfiguration.setVersion( productVersion.toString() );
+
+            try {
+                ProductConfiguration.write(productConfiguration, expandedProductFile);
+
+                if (p2inf.canRead()) {
+                    FileUtils.copyFile(p2inf, new File(expandedProductFile.getParentFile(), p2inf.getName()));
+                }
+            } catch (IOException e) {
+                throw new MojoExecutionException("Error writing expanded product configuration file", e);
             }
-        }
 
-        // String version = getTychoProjectFacet().getArtifactKey( project ).getVersion();
-        // String productVersion = VersioningHelper.getExpandedVersion( project, version );
-        // productConfiguration.setVersion( productVersion.toString() );
-
-        try {
-            ProductConfiguration.write(productConfiguration, expandedProductFile);
-
-            if (p2inf.canRead()) {
-                FileUtils.copyFile(p2inf, new File(expandedProductFile.getParentFile(), p2inf.getName()));
+            if (!createProductArchive || environments != null) {
+                project.getArtifact().setFile(expandedProductFile);
             }
-        } catch (IOException e) {
-            throw new MojoExecutionException("Error writing expanded product configuration file", e);
-        }
-
-        if (!createProductArchive || environments != null) {
-            project.getArtifact().setFile(expandedProductFile);
         }
     }
 
     private ArtifactDependencyWalker getDependencyWalker(TargetEnvironment environment) {
-        return getTychoProjectFacet(PackagingType.TYPE_ECLIPSE_APPLICATION).getDependencyWalker(project, environment);
+        return getTychoProjectFacet(PackagingType.TYPE_ECLIPSE_APPLICATION)
+                .getDependencyWalker(DefaultReactorProject.adapt(project), environment);
     }
 
     private List<TargetEnvironment> getEnvironments() {
@@ -225,7 +233,7 @@ public class ProductExportMojo extends AbstractTychoPackagingMojo {
             return Arrays.asList(environments);
         }
 
-        return TychoProjectUtils.getTargetPlatformConfiguration(project).getEnvironments();
+        return TychoProjectUtils.getTargetPlatformConfiguration(DefaultReactorProject.adapt(project)).getEnvironments();
     }
 
     private File getTarget(TargetEnvironment environment) {
@@ -281,8 +289,6 @@ public class ProductExportMojo extends AbstractTychoPackagingMojo {
         Properties rootProperties = new Properties();
         try (FileInputStream stream = new FileInputStream(new File(project.getBasedir(), generatedBuildProperties))) {
             rootProperties.load(stream);
-        } catch (FileNotFoundException e) {
-            throw new MojoExecutionException("Error including root files for product", e);
         } catch (IOException e) {
             throw new MojoExecutionException("Error including root files for product", e);
         }
@@ -446,14 +452,10 @@ public class ProductExportMojo extends AbstractTychoPackagingMojo {
         configsFolder.mkdirs();
 
         File configIni = new File(configsFolder, "config.ini");
-        FileOutputStream fos = null;
-        try {
-            fos = new FileOutputStream(configIni);
+        try (FileOutputStream fos = new FileOutputStream(configIni)) {
             props.store(fos, "Product Runtime Configuration File");
         } catch (IOException e) {
             throw new MojoExecutionException("Error creating .eclipseproduct file.", e);
-        } finally {
-            IOUtil.close(fos);
         }
     }
 
@@ -531,7 +533,7 @@ public class ProductExportMojo extends AbstractTychoPackagingMojo {
             throw new MojoExecutionException("Native launcher is not found for " + environment.toString());
         }
 
-        File location = artifact.getLocation();
+        File location = artifact.getLocation(true);
 
         String os = environment.getOs();
         String ws = environment.getWs();
@@ -677,24 +679,18 @@ public class ProductExportMojo extends AbstractTychoPackagingMojo {
     }
 
     private void writeStringToFile(File iniFile, String string) throws IOException {
-        OutputStream os = new BufferedOutputStream(new FileOutputStream(iniFile));
-        try {
+        try (OutputStream os = new BufferedOutputStream(new FileOutputStream(iniFile))) {
             IOUtil.copy(string, os);
-        } finally {
-            IOUtil.close(os);
         }
     }
 
     private StringBuffer readFileToString(File iniFile) throws IOException {
-        InputStream is = new BufferedInputStream(new FileInputStream(iniFile));
-        try {
+        try (InputStream is = new BufferedInputStream(new FileInputStream(iniFile))) {
             StringWriter buffer = new StringWriter();
 
             IOUtil.copy(is, buffer, "UTF-8");
 
             return buffer.getBuffer();
-        } finally {
-            IOUtil.close(is);
         }
     }
 
