@@ -1,9 +1,11 @@
 /*******************************************************************************
- * Copyright (c) 2008, 2019 Sonatype Inc. and others.
- * All rights reserved. This program and the accompanying materials
- * are made available under the terms of the Eclipse Public License v1.0
+ * Copyright (c) 2008, 2020 Sonatype Inc. and others.
+ * This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License 2.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/epl-v10.html
+ * https://www.eclipse.org/legal/epl-2.0/
+ *
+ * SPDX-License-Identifier: EPL-2.0
  *
  * Contributors:
  *    Sonatype Inc. - initial API and implementation
@@ -18,6 +20,7 @@ import java.io.IOException;
 import java.io.InputStream;
 import java.io.OutputStreamWriter;
 import java.io.Writer;
+import java.nio.charset.StandardCharsets;
 import java.util.ArrayList;
 import java.util.Collections;
 import java.util.Iterator;
@@ -30,6 +33,7 @@ import java.util.Set;
 import java.util.StringTokenizer;
 
 import org.apache.maven.artifact.Artifact;
+import org.apache.maven.execution.MavenSession;
 import org.apache.maven.model.Build;
 import org.apache.maven.model.Model;
 import org.apache.maven.model.Parent;
@@ -43,14 +47,16 @@ import org.apache.maven.plugin.MojoFailureException;
 import org.apache.maven.plugins.annotations.Component;
 import org.apache.maven.plugins.annotations.Mojo;
 import org.apache.maven.plugins.annotations.Parameter;
+import org.apache.maven.toolchain.ToolchainManager;
+import org.codehaus.plexus.logging.Logger;
 import org.codehaus.plexus.util.ReaderFactory;
 import org.codehaus.plexus.util.xml.XmlStreamReader;
 import org.codehaus.plexus.util.xml.Xpp3Dom;
 import org.codehaus.plexus.util.xml.Xpp3DomBuilder;
 import org.codehaus.plexus.util.xml.pull.XmlPullParserException;
+import org.eclipse.osgi.container.ModuleContainer;
+import org.eclipse.osgi.container.ModuleRevision;
 import org.eclipse.osgi.framework.util.FilePath;
-import org.eclipse.osgi.service.resolver.BundleDescription;
-import org.eclipse.osgi.service.resolver.State;
 import org.eclipse.tycho.ArtifactDescriptor;
 import org.eclipse.tycho.ArtifactKey;
 import org.eclipse.tycho.ArtifactType;
@@ -66,6 +72,7 @@ import org.eclipse.tycho.model.FeatureRef;
 import org.eclipse.tycho.model.PluginRef;
 import org.eclipse.tycho.model.UpdateSite;
 import org.osgi.framework.BundleException;
+import org.osgi.framework.wiring.BundleRevision;
 
 /**
  * Traverse the current directory to find eclipse-plugin/bundle, feature, update site (site.xml) or
@@ -76,8 +83,9 @@ import org.osgi.framework.BundleException;
  * during build.
  * 
  */
-@Mojo(name = "generate-poms", requiresProject = false)
+@Mojo(name = "generate-poms", requiresProject = false, threadSafe = true)
 public class GeneratePomsMojo extends AbstractMojo {
+    private static final Object LOCK = new Object();
 
     private static final class DirectoryFilter implements FileFilter {
         @Override
@@ -104,6 +112,9 @@ public class GeneratePomsMojo extends AbstractMojo {
     @Parameter(property = "baseDir", defaultValue = "${basedir}", required = true)
     private File baseDir;
 
+    @Parameter(property = "session", readonly = true)
+    private MavenSession session;
+
     /**
      * Additional directories to be traversed recursively when searching for projects.
      */
@@ -192,110 +203,118 @@ public class GeneratePomsMojo extends AbstractMojo {
 
     private DefaultDependencyArtifacts platform = new DefaultDependencyArtifacts();
 
+    @Component
+    private ToolchainManager toolchainManager;
+
+    @Component
+    private Logger logger;
+
     @Override
     public void execute() throws MojoExecutionException, MojoFailureException {
-        List<File> baseDirs = getBaseDirs();
-        if (getLog().isDebugEnabled()) {
-            StringBuilder sb = new StringBuilder();
-            sb.append("baseDir=").append(toString(baseDir)).append('\n');
-            sb.append("extraDirs=").append(extraDirs).append('\n');
-            for (int i = 0; i < baseDirs.size(); i++) {
-                sb.append("dir[").append(i).append("]=").append(toString(baseDirs.get(i))).append('\n');
+        synchronized (LOCK) {
+            List<File> baseDirs = getBaseDirs();
+            if (getLog().isDebugEnabled()) {
+                StringBuilder sb = new StringBuilder();
+                sb.append("baseDir=").append(toString(baseDir)).append('\n');
+                sb.append("extraDirs=").append(extraDirs).append('\n');
+                for (int i = 0; i < baseDirs.size(); i++) {
+                    sb.append("dir[").append(i).append("]=").append(toString(baseDirs.get(i))).append('\n');
+                }
+                getLog().debug(sb.toString());
             }
-            getLog().debug(sb.toString());
-        }
 
-        // find all candidate folders
-        List<File> candidateDirs = new ArrayList<>();
-        for (File basedir : baseDirs) {
-            findAndAddCandidates(candidateDirs, basedir);
-        }
-
-        // find all root projects
-        List<File> rootProjects = getRootProjects();
-        if (getLog().isDebugEnabled()) {
-            StringBuilder sb = new StringBuilder();
-            sb.append("rootProjects=").append(this.rootProjects);
-            for (int i = 0; i < rootProjects.size(); i++) {
-                sb.append("rootProject[").append(i).append("]=").append(toString(rootProjects.get(i))).append('\n');
+            // find all candidate folders
+            List<File> candidateDirs = new ArrayList<>();
+            for (File basedir : baseDirs) {
+                findAndAddCandidates(candidateDirs, basedir);
             }
-            getLog().debug(sb.toString());
-        }
 
-        for (File dir : candidateDirs) {
-            if (isPluginProject(dir)) {
-                OsgiManifest mf = bundleReader.loadManifest(dir);
-                platform.addArtifactFile(mf.toArtifactKey(), dir, null);
+            // find all root projects
+            List<File> rootProjects = getRootProjects();
+            if (getLog().isDebugEnabled()) {
+                StringBuilder sb = new StringBuilder();
+                sb.append("rootProjects=").append(this.rootProjects);
+                for (int i = 0; i < rootProjects.size(); i++) {
+                    sb.append("rootProject[").append(i).append("]=").append(toString(rootProjects.get(i))).append('\n');
+                }
+                getLog().debug(sb.toString());
             }
-        }
 
-        // testSuite
-        File testSuiteLocation = null;
-        if (testSuite != null) {
-            ArtifactDescriptor bundle = platform.getArtifact(ArtifactType.TYPE_ECLIPSE_PLUGIN, testSuite, null);
-            if (bundle != null) {
-                testSuiteLocation = bundle.getLocation();
-            }
-        }
-
-        Set<File> projects = new LinkedHashSet<>();
-
-        // always add baseDir
-        projects.add(baseDirs.get(0));
-
-        if (rootProjects.size() > 0) {
-            if (testSuiteLocation != null) {
-                rootProjects.add(testSuiteLocation);
-            }
-            for (File rootProject : rootProjects) {
-                getLog().info("Resolving root project " + toString(rootProject));
-                if (isUpdateSiteProject(rootProject)) {
-                    projects.addAll(getSiteFeaturesAndPlugins(rootProject));
-                    projects.add(rootProject);
-                } else if (isFeatureProject(rootProject)) {
-                    projects.addAll(getFeatureFeaturesAndPlugins(rootProject));
-                    projects.add(rootProject);
-                } else if (isPluginProject(rootProject)) {
-                    addPluginImpl(projects, rootProject); // TODO getPluginAndDependencies
-                    projects.add(rootProject);
-                } else {
-                    getLog().warn("Unsupported root project " + toString(rootProject));
+            for (File dir : candidateDirs) {
+                if (isPluginProject(dir)) {
+                    OsgiManifest mf = bundleReader.loadManifest(dir);
+                    platform.addArtifactFile(mf.toArtifactKey(), dir, null);
                 }
             }
-        } else {
-            projects.addAll(candidateDirs);
-        }
 
-        if (getLog().isDebugEnabled()) {
-            getLog().debug("Collected " + projects.size() + " projects");
-            for (File dir : projects) {
-                getLog().debug("\t" + toString(dir));
+            // testSuite
+            File testSuiteLocation = null;
+            if (testSuite != null) {
+                ArtifactDescriptor bundle = platform.getArtifact(ArtifactType.TYPE_ECLIPSE_PLUGIN, testSuite, null);
+                if (bundle != null) {
+                    testSuiteLocation = bundle.getLocation(true);
+                }
             }
-        }
 
-        // write poms
-        Iterator<File> projectIter = projects.iterator();
-        File parentDir = projectIter.next();
-        if (!projectIter.hasNext()) {
-            if (isProjectDir(parentDir)) {
-                generatePom(null, parentDir);
+            Set<File> projects = new LinkedHashSet<>();
+
+            // always add baseDir
+            projects.add(baseDirs.get(0));
+
+            if (!rootProjects.isEmpty()) {
+                if (testSuiteLocation != null) {
+                    rootProjects.add(testSuiteLocation);
+                }
+                for (File rootProject : rootProjects) {
+                    getLog().info("Resolving root project " + toString(rootProject));
+                    if (isUpdateSiteProject(rootProject)) {
+                        projects.addAll(getSiteFeaturesAndPlugins(rootProject));
+                        projects.add(rootProject);
+                    } else if (isFeatureProject(rootProject)) {
+                        projects.addAll(getFeatureFeaturesAndPlugins(rootProject));
+                        projects.add(rootProject);
+                    } else if (isPluginProject(rootProject)) {
+                        addPluginImpl(projects, rootProject); // TODO getPluginAndDependencies
+                        projects.add(rootProject);
+                    } else {
+                        getLog().warn("Unsupported root project " + toString(rootProject));
+                    }
+                }
             } else {
-                throw new MojoExecutionException("Could not find any valid projects");
+                projects.addAll(candidateDirs);
             }
-        } else {
-            Model parent = readPomTemplate("parent-pom.xml");
-            parent.setGroupId(groupId);
-            parent.setArtifactId(parentDir.getName());
-            parent.setVersion(version);
-            while (projectIter.hasNext()) {
-                File projectDir = projectIter.next();
-                generatePom(parent, projectDir);
-                parent.addModule(getModuleName(parentDir, projectDir));
+
+            if (getLog().isDebugEnabled()) {
+                getLog().debug("Collected " + projects.size() + " projects");
+                for (File dir : projects) {
+                    getLog().debug("\t" + toString(dir));
+                }
+            }
+
+            // write poms
+            Iterator<File> projectIter = projects.iterator();
+            File parentDir = projectIter.next();
+            if (!projectIter.hasNext()) {
+                if (isProjectDir(parentDir)) {
+                    generatePom(null, parentDir);
+                } else {
+                    throw new MojoExecutionException("Could not find any valid projects");
+                }
+            } else {
+                Model parent = readPomTemplate("parent-pom.xml");
+                parent.setGroupId(groupId);
+                parent.setArtifactId(parentDir.getName());
+                parent.setVersion(version);
+                while (projectIter.hasNext()) {
+                    File projectDir = projectIter.next();
+                    generatePom(parent, projectDir);
+                    parent.addModule(getModuleName(parentDir, projectDir));
+                }
+                reorderModules(parent, parentDir, testSuiteLocation);
+                addTychoExtension(parent);
+                writePom(parentDir, parent);
+                generateAggregatorPoms(testSuiteLocation);
             }
-            reorderModules(parent, parentDir, testSuiteLocation);
-            addTychoExtension(parent);
-            writePom(parentDir, parent);
-            generateAggregatorPoms(testSuiteLocation);
         }
     }
 
@@ -412,7 +431,7 @@ public class GeneratePomsMojo extends AbstractMojo {
             File basedir = updateSite.getKey();
             Model parent = updateSite.getValue();
             Set<File> modules = getSiteFeaturesAndPlugins(basedir);
-            if (aggregator && modules.size() > 0) {
+            if (aggregator && !modules.isEmpty()) {
                 Model modela = readPomTemplate("update-site-poma.xml");
                 setParentOrAddTychoExtension(basedir, modela, parent);
                 modela.setGroupId(groupId);
@@ -614,15 +633,14 @@ public class GeneratePomsMojo extends AbstractMojo {
         if (result.add(basedir)) {
             try {
                 StandardExecutionEnvironment ee = ExecutionEnvironmentUtils
-                        .getExecutionEnvironment(executionEnvironment);
-                State state = resolver.newResolvedState(basedir, ee, platform);
-                BundleDescription bundle = state.getBundleByLocation(basedir.getAbsolutePath());
+                        .getExecutionEnvironment(executionEnvironment, toolchainManager, session, logger);
+                ModuleContainer state = resolver.newResolvedState(basedir, session, ee, platform);
+                ModuleRevision bundle = state.getModule(basedir.getAbsolutePath()).getCurrentRevision();
                 if (bundle != null) {
-                    for (DependencyComputer.DependencyEntry entry : dependencyComputer
-                            .computeDependencies(state.getStateHelper(), bundle)) {
-                        BundleDescription supplier = entry.desc;
-                        File suppliedDir = new File(supplier.getLocation());
-                        if (supplier.getHost() == null && isModuleDir(suppliedDir)) {
+                    for (DependencyComputer.DependencyEntry entry : dependencyComputer.computeDependencies(bundle)) {
+                        ModuleRevision supplier = entry.module;
+                        File suppliedDir = (File) supplier.getRevisionInfo();
+                        if (((supplier.getTypes() & BundleRevision.TYPE_FRAGMENT) == 0) && isModuleDir(suppliedDir)) {
                             addPlugin(result, suppliedDir.getName());
                         }
                     }
@@ -751,7 +769,8 @@ public class GeneratePomsMojo extends AbstractMojo {
 
     private void writePom(File dir, String filename, Model model) throws MojoExecutionException {
         try {
-            try (Writer writer = new OutputStreamWriter(new FileOutputStream(new File(dir, filename)), "UTF-8")) {
+            try (Writer writer = new OutputStreamWriter(new FileOutputStream(new File(dir, filename)),
+                    StandardCharsets.UTF_8)) {
                 modelWriter.write(writer, model);
             }
         } catch (IOException e) {
