@@ -1,15 +1,18 @@
 /*******************************************************************************
- * Copyright (c) 2008, 2019 Sonatype Inc. and others.
- * All rights reserved. This program and the accompanying materials
- * are made available under the terms of the Eclipse Public License v1.0
+ * Copyright (c) 2008, 2020 Sonatype Inc. and others.
+ * This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License 2.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/epl-v10.html
+ * https://www.eclipse.org/legal/epl-2.0/
+ *
+ * SPDX-License-Identifier: EPL-2.0
  *
  * Contributors:
  *    Sonatype Inc. - initial API and implementation
  *    SAP SE - port to surefire 2.10
  *    Mickael Istria (Red Hat Inc.) - 386988 Support for provisioned applications
  *    Bachmann electrontic GmbH - 510425 parallel mode requires threadCount>1 or useUnlimitedThreads=true
+ *    Christoph Läubrich - improve error message in case of failures
  ******************************************************************************/
 package org.eclipse.tycho.surefire;
 
@@ -67,6 +70,7 @@ import org.eclipse.sisu.equinox.launching.internal.EquinoxLaunchConfiguration;
 import org.eclipse.tycho.ArtifactDescriptor;
 import org.eclipse.tycho.ArtifactKey;
 import org.eclipse.tycho.ArtifactType;
+import org.eclipse.tycho.BuildDirectory;
 import org.eclipse.tycho.ReactorProject;
 import org.eclipse.tycho.artifacts.DependencyArtifacts;
 import org.eclipse.tycho.core.BundleProject;
@@ -94,7 +98,6 @@ import org.eclipse.tycho.surefire.provider.impl.ProviderHelper;
 import org.eclipse.tycho.surefire.provider.spi.TestFrameworkProvider;
 import org.eclipse.tycho.surefire.provisioning.ProvisionedInstallationBuilder;
 import org.eclipse.tycho.surefire.provisioning.ProvisionedInstallationBuilderFactory;
-import org.osgi.framework.Version;
 
 /**
  * <p>
@@ -105,24 +108,32 @@ import org.osgi.framework.Version;
  * runtime" consists of the bundle built in this project and its transitive dependencies, plus some
  * Equinox and test harness bundles. The bundles are resolved from the target platform of the
  * project. Note that the test runtime does typically <em>not</em> contain the entire target
- * platform. If there are implicitly required bundles (e.g. <tt>org.eclipse.equinox.ds</tt> to make
+ * platform. If there are implicitly required bundles (e.g. <tt>org.apache.felix.scr</tt> to make
  * declarative services work), they need to be added manually through an <tt>extraRequirements</tt>
  * configuration on the <tt>target-platform-configuration</tt> plugin.
  * </p>
  */
-@Mojo(name = "test", defaultPhase = LifecyclePhase.INTEGRATION_TEST, requiresDependencyResolution = ResolutionScope.RUNTIME)
+@Mojo(name = "test", defaultPhase = LifecyclePhase.INTEGRATION_TEST, requiresDependencyResolution = ResolutionScope.RUNTIME, threadSafe = true)
 public class TestMojo extends AbstractMojo {
 
+    private static String[] UNIX_SIGNAL_NAMES = { "not a signal", // padding, singles start with 1
+            "SIGHUP", "SIGINT", "SIGQUIT", "SIGILL", "SIGTRAP", "SIGABRT", "SIGBUS", "SIGFPE", "SIGKILL", "SIGUSR1",
+            "SIGSEGV", "SIGUSR2", "SIGPIPE", "SIGALRM", "SIGTERM", "SIGSTKFLT", "SIGCHLD", "SIGCONT", "SIGSTOP",
+            "SIGTSTP", "SIGTTIN", "SIGTTOU", "SIGURG", "SIGXCPU", "SIGXFSZ", "SIGVTALRM", "SIGPROF", "SIGWINCH",
+            "SIGIO", "SIGPWR", "SIGSYS" };
+
+    private static final Object LOCK = new Object();
+
     /**
      * Root directory (<a href=
-     * "http://help.eclipse.org/indigo/topic/org.eclipse.platform.doc.isv/reference/misc/runtime-options.html#osgiinstallarea"
+     * "https://help.eclipse.org/indigo/topic/org.eclipse.platform.doc.isv/reference/misc/runtime-options.html#osgiinstallarea"
      * >osgi.install.area</a>) of the Equinox runtime used to execute tests.
      */
     @Parameter(defaultValue = "${project.build.directory}/work")
     private File work;
     /**
      * <a href=
-     * "http://help.eclipse.org/juno/topic/org.eclipse.platform.doc.isv/reference/misc/runtime-options.html#osgiinstancearea"
+     * "https://help.eclipse.org/juno/topic/org.eclipse.platform.doc.isv/reference/misc/runtime-options.html#osgiinstancearea"
      * >OSGi data directory</a> (<code>osgi.instance.area</code>, aka the workspace) of the Equinox
      * runtime used to execute tests.
      */
@@ -150,7 +161,7 @@ public class TestMojo extends AbstractMojo {
      * List of patterns (separated by commas) used to specify the tests that should be included in
      * testing. When not specified and when the <code>test</code> parameter is not specified, the
      * default includes will be
-     * <code>**&#47;Test*.java   **&#47;*Test.java   **&#47;*TestCase.java</code>
+     * <code>**&#47;Test*.java   **&#47;*Test.java   **&#47;*Tests.java   **&#47;*TestCase.java</code>
      */
     @Parameter
     private List<String> includes;
@@ -253,11 +264,11 @@ public class TestMojo extends AbstractMojo {
     /**
      * Additional dependencies to be added to the test runtime. Ignored if {@link #testRuntime} is
      * <code>p2Installed</code>.
-     * 
+     *
      * Note: This parameter has only limited support for dependencies to artifacts within the
      * reactor. Therefore it is recommended to specify <tt>extraRequirements</tt> on the
      * <tt>target-platform-configuration</tt> plugin instead. Example:
-     * 
+     *
      * <pre>
      * &lt;plugin&gt;
      *    &lt;groupId&gt;org.eclipse.tycho&lt;/groupId&gt;
@@ -276,7 +287,7 @@ public class TestMojo extends AbstractMojo {
      *    &lt;/configuration&gt;
      * &lt;/plugin&gt;
      * </pre>
-     * 
+     *
      * The dependencies specified as <tt>extraRequirements</tt> are &ndash; together with the
      * dependencies specified in the <tt>MANIFEST.MF</tt> of the project &ndash; transitively
      * resolved against the target platform. The resulting set of bundles is included in the test
@@ -289,7 +300,7 @@ public class TestMojo extends AbstractMojo {
      * Eclipse application to be run. If not specified, default application
      * org.eclipse.ui.ide.workbench will be used. Application runnable will be invoked from test
      * harness, not directly from Eclipse.
-     * 
+     *
      * Note that you need to ensure that the bundle which defines the configured application is
      * included in the test runtime.
      */
@@ -320,7 +331,7 @@ public class TestMojo extends AbstractMojo {
     /**
      * By default, Tycho Surefire disables JVM assertions for the execution of your test cases. To
      * enable the assertions, set this flag to "true".
-     * 
+     *
      * @since 1.5.0
      */
     @Parameter(property = "enableAssertions", defaultValue = "false")
@@ -352,7 +363,7 @@ public class TestMojo extends AbstractMojo {
      * Identifies a single test (suite) class to run. This is useful if you have a single JUnit test
      * suite class defining which tests should be executed. Will be ignored if {@link #test} is
      * specified. Example:
-     * 
+     *
      * <pre>
      * &lt;testClass&gt;foo.bar.FooTest&lt;/testClass&gt;
      * </pre>
@@ -390,7 +401,7 @@ public class TestMojo extends AbstractMojo {
     /**
      * Bundle start level and auto start configuration used by the test runtime. Ignored if
      * {@link #testRuntime} is <code>p2Installed</code>. Example:
-     * 
+     *
      * <pre>
      * &lt;bundleStartLevel&gt;
      *   &lt;bundle&gt;
@@ -408,7 +419,7 @@ public class TestMojo extends AbstractMojo {
      * The default bundle start level and auto start configuration used by the test runtime for
      * bundles where the start level/auto start is not configured in {@link #bundleStartLevel}.
      * Ignored if {@link #testRuntime} is <code>p2Installed</code>. Example:
-     * 
+     *
      * <pre>
      *   &lt;defaultStartLevel&gt;
      *     &lt;level&gt;6&lt;/level&gt;
@@ -464,10 +475,10 @@ public class TestMojo extends AbstractMojo {
      * Normally tycho will automatically determine the test framework provider based on the test
      * project's classpath. Use this to force using a test framework provider implementation with
      * the given role hint. Tycho comes with providers
-     * &quot;junit3&quot;,&quot;junit4&quot;,&quot;junit47&quot;. Note that when specifying a
-     * providerHint, you have to make sure the provider is actually available in the dependencies of
-     * tycho-surefire-plugin.
-     * 
+     * &quot;junit3&quot;,&quot;junit4&quot;,&quot;junit47&quot;,&quot;junit5&quot;. Note that when
+     * specifying a providerHint, you have to make sure the provider is actually available in the
+     * dependencies of tycho-surefire-plugin.
+     *
      * @since 0.16.0
      */
     @Parameter
@@ -477,7 +488,7 @@ public class TestMojo extends AbstractMojo {
      * Defines the order the tests will be run in. Supported values are "alphabetical",
      * "reversealphabetical", "random", "hourly" (alphabetical on even hours, reverse alphabetical
      * on odd hours) and "filesystem".
-     * 
+     *
      * @since 0.19.0
      */
     @Parameter(defaultValue = "filesystem")
@@ -494,7 +505,7 @@ public class TestMojo extends AbstractMojo {
     /**
      * (JUnit 4.7 provider) Supports values "classes"/"methods"/"both" to run in separate threads,
      * as controlled by threadCount.
-     * 
+     *
      * @since 0.16.0
      */
     @Parameter(property = "parallel")
@@ -502,7 +513,7 @@ public class TestMojo extends AbstractMojo {
 
     /**
      * (JUnit 4.7 provider) Indicates that threadCount is per cpu core.
-     * 
+     *
      * @since 0.16.0
      */
     @Parameter(property = "perCoreThreadCount", defaultValue = "true")
@@ -512,7 +523,7 @@ public class TestMojo extends AbstractMojo {
      * (JUnit 4.7 provider) The attribute thread-count allows you to specify how many threads should
      * be allocated for this execution. Only makes sense to use in conjunction with the parallel
      * parameter.
-     * 
+     *
      * @since 0.16.0
      */
     @Parameter(property = "threadCount")
@@ -522,7 +533,7 @@ public class TestMojo extends AbstractMojo {
      * (JUnit 4.7 provider) Indicates that the thread pool will be unlimited. The parallel parameter
      * and the actual number of classes/methods will decide. Setting this to "true" effectively
      * disables perCoreThreadCount and threadCount.
-     * 
+     *
      * @since 0.16.0
      */
     @Parameter(property = "useUnlimitedThreads", defaultValue = "false")
@@ -530,7 +541,7 @@ public class TestMojo extends AbstractMojo {
 
     /**
      * Use this to specify surefire provider-specific properties.
-     * 
+     *
      * @since 0.16.0
      */
     @Parameter
@@ -553,9 +564,9 @@ public class TestMojo extends AbstractMojo {
      * with a certain performance overhead for executing the provisioning operations otherwise not
      * required.</li>
      * </ul>
-     * 
+     *
      * Example configuration which will install product IU under test "example.product.id" using p2:
-     * 
+     *
      * <pre>
      * &lt;plugin&gt;
      *    &lt;groupId&gt;org.eclipse.tycho&lt;/groupId&gt;
@@ -583,7 +594,7 @@ public class TestMojo extends AbstractMojo {
      *    &lt;/configuration&gt;
      * &lt;/plugin&gt;
      * </pre>
-     * 
+     *
      * @since 0.19.0
      */
     @Parameter(defaultValue = "default")
@@ -591,13 +602,13 @@ public class TestMojo extends AbstractMojo {
 
     /**
      * p2 <a href=
-     * "http://help.eclipse.org/kepler/index.jsp?topic=%2Forg.eclipse.platform.doc.isv%2Fguide%2Fp2_director.html"
+     * "https://help.eclipse.org/kepler/index.jsp?topic=%2Forg.eclipse.platform.doc.isv%2Fguide%2Fp2_director.html"
      * >profile</a> name of the installation under test.
-     * 
+     *
      * Only relevant if {@link #testRuntime} is <code>p2Installed</code>. If tests are installed on
      * top of an already existing installation in {@link #work}, this must match the name of the
      * existing profile.
-     * 
+     *
      * @since 0.19.0
      */
     // default value should be kept the same as DirectorMojo#profile default value
@@ -633,16 +644,16 @@ public class TestMojo extends AbstractMojo {
      * The value of BREE will be matched against the id of the toolchain elements in
      * toolchains.xml.</li>
      * </ul>
-     * 
+     *
      * Example for BREE: <br>
      * In <code>META-INF/MANIFEST.MF</code>:
-     * 
+     *
      * <pre>
      * Bundle-RequiredExecutionEnvironment: JavaSE-1.7
      * </pre>
-     * 
+     *
      * In toolchains.xml:
-     * 
+     *
      * <pre>
      * &lt;toolchains&gt;
      *    &lt;toolchain&gt;
@@ -665,7 +676,7 @@ public class TestMojo extends AbstractMojo {
      * suite files. The suite files will overwrite the {@link #includes} and {@link #excludes}
      * patterns. The path to the suite file(s) could be relative (test bundle classpath) or an
      * absolute path to xml files outside the test bundle.
-     * 
+     *
      * <pre>
      * &lt;configuration&gt;
      *   &lt;suiteXmlFiles&gt;
@@ -689,17 +700,19 @@ public class TestMojo extends AbstractMojo {
             return;
         }
 
-        EquinoxInstallation equinoxTestRuntime;
-        if ("p2Installed".equals(testRuntime)) {
-            equinoxTestRuntime = createProvisionedInstallation();
-        } else if ("default".equals(testRuntime)) {
-            equinoxTestRuntime = createEclipseInstallation();
-        } else {
-            throw new MojoExecutionException("Configured testRuntime parameter value '" + testRuntime
-                    + "' is unkown. Allowed values: 'default', 'p2Installed'.");
-        }
+        synchronized (LOCK) {
+            EquinoxInstallation equinoxTestRuntime;
+            if ("p2Installed".equals(testRuntime)) {
+                equinoxTestRuntime = createProvisionedInstallation();
+            } else if ("default".equals(testRuntime)) {
+                equinoxTestRuntime = createEclipseInstallation();
+            } else {
+                throw new MojoExecutionException("Configured testRuntime parameter value '" + testRuntime
+                        + "' is unkown. Allowed values: 'default', 'p2Installed'.");
+            }
 
-        runTest(equinoxTestRuntime);
+            runTest(equinoxTestRuntime);
+        }
     }
 
     protected boolean shouldSkip() {
@@ -726,8 +739,9 @@ public class TestMojo extends AbstractMojo {
 
     private EquinoxInstallation createProvisionedInstallation() throws MojoExecutionException {
         try {
-            TestFrameworkProvider provider = providerHelper.selectProvider(getProjectType().getClasspath(project),
-                    getMergedProviderProperties(), providerHint);
+            TestFrameworkProvider provider = providerHelper.selectProvider(
+                    getProjectType().getClasspath(DefaultReactorProject.adapt(project)), getMergedProviderProperties(),
+                    providerHint);
             createSurefireProperties(provider);
 
             ProvisionedInstallationBuilder installationBuilder = provisionedInstallationBuilderFactory
@@ -759,7 +773,8 @@ public class TestMojo extends AbstractMojo {
         // 2. test harness bundles
         iusToInstall.addAll(providerHelper.getSymbolicNames(testHarnessArtifacts));
         // 3. extra dependencies
-        for (Dependency extraDependency : TychoProjectUtils.getTargetPlatformConfiguration(project)
+        for (Dependency extraDependency : TychoProjectUtils
+                .getTargetPlatformConfiguration(DefaultReactorProject.adapt(project))
                 .getDependencyResolverConfiguration().getExtraRequirements()) {
             String type = extraDependency.getType();
             if (ArtifactType.TYPE_ECLIPSE_PLUGIN.equals(type) || ArtifactType.TYPE_INSTALLABLE_UNIT.equals(type)) {
@@ -812,8 +827,9 @@ public class TestMojo extends AbstractMojo {
             }
         }
 
-        TestFrameworkProvider provider = providerHelper.selectProvider(getProjectType().getClasspath(project),
-                getMergedProviderProperties(), providerHint);
+        TestFrameworkProvider provider = providerHelper.selectProvider(
+                getProjectType().getClasspath(DefaultReactorProject.adapt(project)), getMergedProviderProperties(),
+                providerHint);
         createSurefireProperties(provider);
         for (ArtifactDescriptor artifact : testRuntimeArtifacts.getArtifacts(ArtifactType.TYPE_ECLIPSE_PLUGIN)) {
             // note that this project is added as directory structure rooted at project basedir.
@@ -859,7 +875,8 @@ public class TestMojo extends AbstractMojo {
         if (this.dependencies != null) {
             dependencies.addAll(Arrays.asList(this.dependencies));
         }
-        TargetPlatformConfiguration configuration = TychoProjectUtils.getTargetPlatformConfiguration(project);
+        TargetPlatformConfiguration configuration = TychoProjectUtils
+                .getTargetPlatformConfiguration(DefaultReactorProject.adapt(project));
         dependencies.addAll(configuration.getDependencyResolverConfiguration().getExtraRequirements());
         dependencies.addAll(getTestDependencies());
         return dependencies;
@@ -958,7 +975,8 @@ public class TestMojo extends AbstractMojo {
     }
 
     protected ScanResult scanForTests() {
-        List<String> defaultIncludes = Arrays.asList("**/Test*.class", "**/*Test.class", "**/*TestCase.class");
+        List<String> defaultIncludes = Arrays.asList("**/Test*.class", "**/*Test.class", "**/*Tests.class",
+                "**/*TestCase.class");
         List<String> defaultExcludes = Arrays.asList("**/*$*");
         List<String> includeList;
         List<String> excludeList;
@@ -982,7 +1000,7 @@ public class TestMojo extends AbstractMojo {
         } else {
             excludeList = defaultExcludes;
         }
-        // TODO bug 495353 we should we rather let TestListResolver do the work here 
+        // TODO bug 495353 we should we rather let TestListResolver do the work here
         // by passing in the unparsed String or Strings instead of already parsed include/exclude list
         // (this would add support for running single test methods, negation etc.)
         TestListResolver resolver = new TestListResolver(includeList, excludeList);
@@ -1005,13 +1023,15 @@ public class TestMojo extends AbstractMojo {
 
     private void runTest(EquinoxInstallation testRuntime) throws MojoExecutionException, MojoFailureException {
         int result;
+        File logFile = new File(osgiDataDirectory, ".metadata/.log");
+        LaunchConfiguration cli;
         try {
             if (deleteOsgiDataDirectory) {
                 FileUtils.deleteDirectory(osgiDataDirectory);
             }
-            LaunchConfiguration cli = createCommandLine(testRuntime);
-            getLog().info(
-                    "Expected eclipse log file: " + new File(osgiDataDirectory, ".metadata/.log").getAbsolutePath());
+            cli = createCommandLine(testRuntime);
+            getLog().info("Executing Test Runtime with timeout " + forkedProcessTimeoutInSeconds
+                    + ", logs (if any) will be placed at: " + logFile.getAbsolutePath());
             result = launcher.execute(cli, forkedProcessTimeoutInSeconds);
         } catch (Exception e) {
             throw new MojoExecutionException("Error while executing platform", e);
@@ -1052,12 +1072,54 @@ public class TestMojo extends AbstractMojo {
             break;
 
         default:
-            throw new MojoFailureException("An unexpected error occurred while launching the test runtime (return code "
-                    + result + "). See log " + new File(osgiDataDirectory, ".metadata/.log").getAbsolutePath()
-                    + " for details.");
+            StringBuilder defaultMessage = new StringBuilder(
+                    "An unexpected error occurred while launching the test runtime (process returned error code ");
+            defaultMessage.append(decodeReturnCode(result));
+            defaultMessage.append(").");
+            if (logFile.exists()) {
+                defaultMessage.append(" The process logfile ");
+                defaultMessage.append(logFile.getAbsolutePath());
+                defaultMessage.append(" might contain further details.");
+            }
+            defaultMessage.append(" Command-line used to launch the sub-process was ");
+            defaultMessage.append(cli.getJvmExecutable());
+            String[] vmArguments = cli.getVMArguments();
+            if (vmArguments != null && vmArguments.length > 0) {
+                defaultMessage.append(" ");
+                defaultMessage.append(String.join(" ", vmArguments));
+            }
+            defaultMessage.append(" -jar ");
+            defaultMessage.append(cli.getLauncherJar());
+            String[] programArguments = cli.getProgramArguments();
+            if (programArguments != null && programArguments.length > 0) {
+                defaultMessage.append(" ");
+                defaultMessage.append(String.join(" ", programArguments));
+            }
+            defaultMessage.append(" in working directory ");
+            defaultMessage.append(cli.getWorkingDirectory());
+            throw new MojoFailureException(defaultMessage.toString());
         }
     }
 
+    private String decodeReturnCode(int result) {
+        try {
+            Properties properties = (Properties) DefaultReactorProject.adapt(project)
+                    .getContextValue(TychoConstants.CTX_MERGED_PROPERTIES);
+            if (PlatformPropertiesUtils.OS_LINUX.equals(PlatformPropertiesUtils.getOS(properties))) {
+                int signal = result - 128;
+                if (signal > 0 && signal < UNIX_SIGNAL_NAMES.length) {
+                    return result + "(" + UNIX_SIGNAL_NAMES[signal] + " received?)";
+                }
+            } else if (PlatformPropertiesUtils.OS_WIN32.equals(PlatformPropertiesUtils.getOS(properties))) {
+                return result + " (HRESULT Code 0x" + Integer.toHexString(result).toUpperCase()
+                        + ", check for example https://www.hresult.info/ for further details)";
+            }
+        } catch (RuntimeException e) {
+            getLog().debug("Decoding returncode failed", e);
+        }
+        return String.valueOf(result);
+    }
+
     protected Toolchain getToolchain() throws MojoExecutionException {
         if (JDKUsage.SYSTEM.equals(useJDK)) {
             if (toolchainManager != null) {
@@ -1065,7 +1127,8 @@ public class TestMojo extends AbstractMojo {
             }
             return null;
         }
-        String profileName = TychoProjectUtils.getExecutionEnvironmentConfiguration(project).getProfileName();
+        String profileName = TychoProjectUtils
+                .getExecutionEnvironmentConfiguration(DefaultReactorProject.adapt(project)).getProfileName();
         Toolchain toolChain = toolchainProvider.findMatchingJavaToolChain(session, profileName);
         if (toolChain == null) {
             throw new MojoExecutionException("useJDK = BREE configured, but no toolchain of type 'jdk' with id '"
@@ -1094,7 +1157,8 @@ public class TestMojo extends AbstractMojo {
 
         cli.addVMArguments("-Dosgi.noShutdown=false");
 
-        Properties properties = (Properties) project.getContextValue(TychoConstants.CTX_MERGED_PROPERTIES);
+        Properties properties = (Properties) DefaultReactorProject.adapt(project)
+                .getContextValue(TychoConstants.CTX_MERGED_PROPERTIES);
         cli.addVMArguments("-Dosgi.os=" + PlatformPropertiesUtils.getOS(properties), //
                 "-Dosgi.ws=" + PlatformPropertiesUtils.getWS(properties), //
                 "-Dosgi.arch=" + PlatformPropertiesUtils.getArch(properties));
@@ -1144,7 +1208,8 @@ public class TestMojo extends AbstractMojo {
     }
 
     private void addCustomProfileArg(EquinoxLaunchConfiguration cli) throws MojoExecutionException {
-        ExecutionEnvironmentConfiguration eeConfig = TychoProjectUtils.getExecutionEnvironmentConfiguration(project);
+        ExecutionEnvironmentConfiguration eeConfig = TychoProjectUtils
+                .getExecutionEnvironmentConfiguration(DefaultReactorProject.adapt(project));
         if (eeConfig.isCustomProfile()) {
             Properties customProfileProps = eeConfig.getFullSpecification().getProfileProperties();
             File profileFile = new File(new File(project.getBuild().getDirectory()), "custom.profile");
@@ -1181,13 +1246,7 @@ public class TestMojo extends AbstractMojo {
 
     private String getTestApplication(EquinoxInstallationDescription testRuntime) {
         if (useUIHarness) {
-            ArtifactDescriptor systemBundle = testRuntime.getSystemBundle();
-            Version osgiVersion = Version.parseVersion(systemBundle.getKey().getVersion());
-            if (osgiVersion.compareTo(EquinoxInstallationDescription.EQUINOX_VERSION_3_3_0) < 0) {
-                return "org.eclipse.tycho.surefire.osgibooter.uitest32";
-            } else {
-                return "org.eclipse.tycho.surefire.osgibooter.uitest";
-            }
+            return "org.eclipse.tycho.surefire.osgibooter.uitest";
         } else {
             return "org.eclipse.tycho.surefire.osgibooter.headlesstest";
         }
@@ -1196,8 +1255,9 @@ public class TestMojo extends AbstractMojo {
     private String getBuildOutputDirectories() {
         StringBuilder sb = new StringBuilder();
         ReactorProject reactorProject = getReactorProject();
-        sb.append(reactorProject.getOutputDirectory());
-        sb.append(',').append(reactorProject.getTestOutputDirectory());
+        BuildDirectory buildDirectory = reactorProject.getBuildDirectory();
+        sb.append(buildDirectory.getOutputDirectory());
+        sb.append(',').append(buildDirectory.getTestOutputDirectory());
         for (BuildOutputJar outputJar : osgiBundle.getEclipsePluginProject(reactorProject).getOutputJars()) {
             if (".".equals(outputJar.getName())) {
                 // handled above
