@@ -1,29 +1,23 @@
 /*******************************************************************************
- * Copyright (c) 2008, 2019 Sonatype Inc. and others.
- * All rights reserved. This program and the accompanying materials
- * are made available under the terms of the Eclipse Public License v1.0
+ * Copyright (c) 2008, 2020 Sonatype Inc. and others.
+ * This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License 2.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/epl-v10.html
+ * https://www.eclipse.org/legal/epl-2.0/
+ *
+ * SPDX-License-Identifier: EPL-2.0
  *
  * Contributors:
  *    Sonatype Inc. - initial API and implementation
  *******************************************************************************/
 package org.eclipse.tycho.test;
 
-import java.io.BufferedInputStream;
-import java.io.BufferedOutputStream;
 import java.io.File;
-import java.io.FileInputStream;
-import java.io.FileOutputStream;
 import java.io.IOException;
-import java.io.InputStream;
-import java.io.OutputStream;
-import java.io.StringWriter;
 
 import org.apache.maven.it.Verifier;
 import org.apache.maven.shared.utils.io.DirectoryScanner;
 import org.codehaus.plexus.util.FileUtils;
-import org.codehaus.plexus.util.IOUtil;
 import org.eclipse.tycho.test.util.EnvironmentUtil;
 import org.junit.Assert;
 import org.junit.Rule;
@@ -109,7 +103,7 @@ public abstract class AbstractTychoIntegrationTest {
         verifier.setLocalRepo(EnvironmentUtil.getLocalRepo());
 
         String customOptions = System.getProperty("it.cliOptions");
-        if (customOptions != null && customOptions.trim().length() > 0) {
+        if (customOptions != null && !customOptions.trim().isEmpty()) {
             verifier.getCliOptions().add(customOptions);
         }
 
@@ -167,7 +161,7 @@ public abstract class AbstractTychoIntegrationTest {
     protected void assertFileExists(File targetdir, String pattern) {
         DirectoryScanner ds = new DirectoryScanner();
         ds.setBasedir(targetdir);
-        ds.setIncludes(new String[] { pattern });
+        ds.setIncludes(pattern);
         ds.scan();
         Assert.assertEquals(targetdir.getAbsolutePath() + "/" + pattern, 1, ds.getIncludedFiles().length);
         Assert.assertTrue(targetdir.getAbsolutePath() + "/" + pattern,
@@ -177,7 +171,7 @@ public abstract class AbstractTychoIntegrationTest {
     protected void assertDirectoryExists(File targetdir, String pattern) {
         DirectoryScanner ds = new DirectoryScanner();
         ds.setBasedir(targetdir);
-        ds.setIncludes(new String[] { pattern });
+        ds.setIncludes(pattern);
         ds.scan();
         Assert.assertEquals(targetdir.getAbsolutePath() + "/" + pattern, 1, ds.getIncludedDirectories().length);
         Assert.assertTrue(targetdir.getAbsolutePath() + "/" + pattern,
@@ -187,7 +181,7 @@ public abstract class AbstractTychoIntegrationTest {
     protected void assertFileDoesNotExist(File targetdir, String pattern) {
         DirectoryScanner ds = new DirectoryScanner();
         ds.setBasedir(targetdir);
-        ds.setIncludes(new String[] { pattern });
+        ds.setIncludes(pattern);
         ds.scan();
         Assert.assertEquals(targetdir.getAbsolutePath() + "/" + pattern, 0, ds.getIncludedFiles().length);
     }
@@ -196,26 +190,4 @@ public abstract class AbstractTychoIntegrationTest {
         return file.getCanonicalFile().toURI().normalize().toString();
     }
 
-    protected void writeStringToFile(File iniFile, String string) throws IOException {
-        OutputStream os = new BufferedOutputStream(new FileOutputStream(iniFile));
-        try {
-            IOUtil.copy(string, os);
-        } finally {
-            IOUtil.close(os);
-        }
-    }
-
-    protected StringBuffer readFileToString(File iniFile) throws IOException {
-        InputStream is = new BufferedInputStream(new FileInputStream(iniFile));
-        try {
-            StringWriter buffer = new StringWriter();
-
-            IOUtil.copy(is, buffer, "UTF-8");
-
-            return buffer.getBuffer();
-        } finally {
-            IOUtil.close(is);
-        }
-    }
-
 }
